   function DecToHex
      parameters intDEC, intPlaces
   	/* Rich - www.autotraker.com

      	Version 1.1 - made sure we were always working with an integer in 
         	addition to a absolute value.

         Integer to hex replacement for ITOH. Returns the hexadecimal equivalent
         of a specified number, as a character string. It is not limited to 
         32-bit integers as ITOH is and always returns the absolute value.

         Syntax:
         DecToHex(<int expN> [, <chars expN>])
         <int expN> The integer whose hexadecimal equivalent to return.
         <chars expN> The minimum number of characters to include in the 
         	returned hexadecimal character string.

         ITOH() is designed to only work with 32-bit integers. 
      	This is a replacement for ITOH() which tops out at 2^32 and then 
         starts to wrap. For example; ? itoh(2^32) will return hex for zero
         and ? itoh(2^32+1) will return the hex for 1 and so on until you hit 
         the next multiple where it would wrap again. This function will not
         top out or wrap and doesn't have that limit. If you are working with 
         only 32-bit integers from something like an EXTERN, you may need to 
         stay with ITOH because of its characteristics.  

         With negitive numbers instead of counting backwards from (2^32) as 
         ITOH() does, it will just return the absolute hex value. 
         ITOH(-1) will return "FFFFFFFF" and ITOH(-2) will return "FFFFFFFE"
         DecToHex(-1) will return hex "1" and DecToHex(-2) will return hex "2" 

      	Based on a VBScript found at 
         http://www.visualbasicscript.com/m_1933/tm.htm
         I did add a few other features and changed it around to work with dBASE. 

         It also allows you to specify the minimum number of characters 
         to include in the returned hexadecimal character string
         See OH for ITOH for more information and syntax.

         nNum = (2^32)-1 	// good in both itoh and DecToHex
         nNum = (2^32) 		// overflow with itoh, good with DecToHex 
         nNum = (2^32)+2 	// overflow with itoh, good with DecToHex 
         nNum = (2^32)*2+2 // overflow with itoh, good with DecToHex 
         ? DecToHex(nNum)
			
         ? DecToHex(123) 	// returns "7B"
         ? DecToHex(123,4) // returns "007B"

      */
      if not type("intPlaces")=="N" or empty(intPlaces)
         intPlaces = 0
      endif
      if not type("intDEC")=="N" or empty(intDEC)
         intDEC = 0
      endif

      strResult = ""
      intValue = 0
      intExp = 0
      strResult = ""

      // make sure we are always working with the absolute value of a number
      // and a integer. 
      intValue = ABS(INT(intDEC))

      // to overcome the limitaions of 65536 and yet have the smallest return
      // we are dynamically building the intExp based on the len of the integer 
      intExp = htoi("1"+replicate("0",len(""+intValue)/2))

      // if intExp is under one then it is a fraction or zero, so get out. 
      do while intExp >= 1
         if intValue >= intExp
         	// at least ITOH() can handle small integers so I don't have
            // to use an array or string to find the right hex 
            strResult += itoh(int(intValue / intExp))
            // get the remainder
            intValue -= intExp * int(intValue / intExp)
         else
         	// less than our intExp, so it has to be an zero
            strResult = strResult + "0"
         endif
         // divide by 16 to give us base 16
         intExp /= 16
      enddo

      // if the result is smaller than we want, we just pad the beginning
      if strResult.length < intPlaces
         strResult = right(replicate("0",intPlaces)+strResult,intPlaces) 
      endif

      return strResult
