// Metric: 6 - Pixels only
/*
   Class PixCalendar22   A custom dBASE Calendar Component Class
   Purpose: Get and set dates on visual calendar control
   Author: A. A. Katz
   Date: 03-18-1997, 04-01-1997
   Copyright: A. A. Katz. All rights reserved. However, this class
      may be used by developers in their applications. They
      may modify source code. They may NOT resell this
      custom class as a component or class.
   Usage: Register this class in dBASE File/Set up Custom Controls
          Drop on form as needed
          To use from dUFLP:
             set procedure to :dUFLP:PixCalendar22.cc additive

   Main Methods: setDate(dDate) sets new calendar default date
                 GetDate()  returns the selected date on the calendar
   Main Property: Date (this.date). Contains the currently selected date.
                              If no date is sent on instantiation,
                              today's date is used
   Upgraded  01/28/2000    AAK  For Improved interprocess messaging
   Added Support for an "owner object" with callback if
        it has a setDate method
   Support for automatic field updates if this.currentfield
        object has a date-type value property
   Fixed  11/12/2002  Uses date class for generating date values
        (instead of ctod).  Now works regardless of MDY,YMD, or DMY setting.
   Metric changed to Pixel                       // Jean-Pierre Martel, 2004-07-17
   French translation                            // Jean-Pierre Martel, 2004-07-17
   German translation                            // Jean-Pierre Martel, 2004-07-17
   Spanish translation                           // Jean-Pierre Martel, 2004-07-17
   Pushbuttons replaced with TextLabels          // Jean-Pierre Martel, 2005-03-21
   Color of the class matches Windows' settings  // Jean-Pierre Martel, 2005-03-21
   onClick() event handler added                 // Jean-Pierre Martel, 2005-04-18
   Revised to fix old selections not removed     // Gerald Lightsey (GKL) 2005-07-10
   Integrated Ken Mayer's holiday setting routine// (GKL) 2005-07-10
   Color constants #define(ed)                   // (GKL) 2005-07-10

   Dependencies: 
      :dUFLP:DateEx.cc
      :dUFLP:Holiday.cc
         :dUFLP:Time.cc
         :dUFLP:Hebrew.cc
         :dUFLP:SetProc.prg
*/

Class PixCalendar22(oF) of Container(oF) Custom
   set procedure to :dUFLP:dateex.cc additive
   set procedure to :dUFLP:holiday.cc additive

   with (this)
      onOpen = class::calendar_OnOpen
      onDesignOpen = {; Form.metric=6 }
      metric = 6  // Pixels
      left = 1
      top = 2
      width = 151
      height = 141
      colorNormal = "BtnFace"
      sizeable = false
      language = "English"
   endwith

   this.aCurrentOwners = new array() // init exclusive owner
   this.aCurrentFields = new array() // update any number of fields
   this.language       = "English"   // or "French" or "German" or "Spanish"

   // instance of dateex class and holiday class:
   this.DateEx        = new DateEx()
   this.Holiday       = new Holiday()

   // Match colors of the application with Windows' settings
   #define COLOR_ACTIVECAPTION     2
   #define COLOR_CAPTIONTEXT       9
   if type('GetSysColor') # 'FP'
      extern culong GetSysColor( cint ) user32
   endif
   #define This_ColorNormal "0x" + itoh(GetSysColor(COLOR_CAPTIONTEXT)) + "/0x" + itoh(GetSysColor(COLOR_ACTIVECAPTION))
	#define DEFAULTCOLOR "BLACK/WHITE"
   #define TODAYCOLOR "RED/SILVER"
   #define HOLIDAYCOLOR "LIME/MAROON"

   this.RECTANGLE1 = new RECTANGLE(this)
   with (this.RECTANGLE1)
      left = 3.0
      top = 3.0
      width = 139.0
      height = 129.0
      text = ""
      colorNormal = "BtnText/White"
      border = false
      borderStyle = 2	// Lowered
   endwith

   this.DAYSDISPLAY = new TEXTLABEL(this)
   with (this.DAYSDISPLAY)
      alignVertical = 1	// Middle
      height = 14
      left = 7
      top = 24
      width = 133
      fontName = 'Arial'
      fontSize = 7
      text = "  S    M    T    W     T    F     S"
      colorNormal = "BtnText/BtnFace"
      pageno = 0
   endwith

   this.TEXTLABEL1 = new TextDate(this)
   with (this.TEXTLABEL1)
      left = 8
      top = 39
   endwith

   this.TEXTLABEL2 = new TextDate(this)
   with (this.TEXTLABEL2)
      left = 27
      top = 39
   endwith

   this.TEXTLABEL3 = new TextDate(this)
   with (this.TEXTLABEL3)
      left = 46
      top = 39
   endwith

   this.TEXTLABEL4 = new TextDate(this)
   with (this.TEXTLABEL4)
      left = 65
      top = 39
   endwith

   this.TEXTLABEL5 = new TextDate(this)
   with (this.TEXTLABEL5)
      left = 84
      top = 39
   endwith

   this.TEXTLABEL6 = new TextDate(this)
   with (this.TEXTLABEL6)
      left = 103
      top = 39
   endwith

   this.TEXTLABEL7 = new TextDate(this)
   with (this.TEXTLABEL7)
      left = 122
      top = 39
   endwith

   this.TEXTLABEL8 = new TextDate(this)
   with (this.TEXTLABEL8)
      left = 8
      top = 54
   endwith

   this.TEXTLABEL9 = new TextDate(this)
   with (this.TEXTLABEL9)
      left = 27
      top = 54
   endwith

   this.TEXTLABEL10 = new TextDate(this)
   with (this.TEXTLABEL10)
      left = 46
      top = 54
   endwith

   this.TEXTLABEL11 = new TextDate(this)
   with (this.TEXTLABEL11)
      left = 65
      top = 54
   endwith

   this.TEXTLABEL12 = new TextDate(this)
   with (this.TEXTLABEL12)
      left = 84
      top = 54
   endwith

   this.TEXTLABEL13 = new TextDate(this)
   with (this.TEXTLABEL13)
      left = 103
      top = 54
   endwith

   this.TEXTLABEL14 = new TextDate(this)
   with (this.TEXTLABEL14)
      left = 122
      top = 54
   endwith

   this.TEXTLABEL15 = new TextDate(this)
   with (this.TEXTLABEL15)
      left = 8
      top = 69
   endwith

   this.TEXTLABEL16 = new TextDate(this)
   with (this.TEXTLABEL16)
      left = 27
      top = 69
   endwith

   this.TEXTLABEL17 = new TextDate(this)
   with (this.TEXTLABEL17)
      left = 46
      top = 69
   endwith

   this.TEXTLABEL18 = new TextDate(this)
   with (this.TEXTLABEL18)
      left = 65
      top = 69
   endwith

   this.TEXTLABEL19 = new TextDate(this)
   with (this.TEXTLABEL19)
      left = 84
      top = 69
   endwith

   this.TEXTLABEL20 = new TextDate(this)
   with (this.TEXTLABEL20)
      left = 103
      top = 69
   endwith

   this.TEXTLABEL21 = new TextDate(this)
   with (this.TEXTLABEL21)
      left = 122
      top = 69
   endwith

   this.TEXTLABEL22 = new TextDate(this)
   with (this.TEXTLABEL22)
      left = 8
      top = 84
   endwith

   this.TEXTLABEL23 = new TextDate(this)
   with (this.TEXTLABEL23)
      left = 27
      top = 84
   endwith

   this.TEXTLABEL24 = new TextDate(this)
   with (this.TEXTLABEL24)
      left = 46
      top = 84
   endwith

   this.TEXTLABEL25 = new TextDate(this)
   with (this.TEXTLABEL25)
      left = 65
      top = 84
   endwith

   this.TEXTLABEL26 = new TextDate(this)
   with (this.TEXTLABEL26)
      left = 84
      top = 84
   endwith

   this.TEXTLABEL27 = new TextDate(this)
   with (this.TEXTLABEL27)
      left = 103
      top = 84
   endwith

   this.TEXTLABEL28 = new TextDate(this)
   with (this.TEXTLABEL28)
      left = 122
      top = 84
   endwith

   this.TEXTLABEL29 = new TextDate(this)
   with (this.TEXTLABEL29)
      left = 8
      top = 99
   endwith

   this.TEXTLABEL30 = new TextDate(this)
   with (this.TEXTLABEL30)
      left = 27
      top = 99
   endwith

   this.TEXTLABEL31 = new TextDate(this)
   with (this.TEXTLABEL31)
      left = 46
      top = 99
   endwith

   this.TEXTLABEL32 = new TextDate(this)
   with (this.TEXTLABEL32)
      left = 65
      top = 99
   endwith

   this.TEXTLABEL33 = new TextDate(this)
   with (this.TEXTLABEL33)
      left = 84
      top = 99
   endwith

   this.TEXTLABEL34 = new TextDate(this)
   with (this.TEXTLABEL34)
      left = 103
      top = 99
   endwith

   this.TEXTLABEL35 = new TextDate(this)
   with (this.TEXTLABEL35)
      left = 122
      top = 99
   endwith

   this.TEXTLABEL36 = new TextDate(this)
   with (this.TEXTLABEL36)
      left = 8
      top = 114
   endwith

   this.TEXTLABEL37 = new TextDate(this)
   with (this.TEXTLABEL37)
      left = 27
      top = 114
   endwith

   this.MONTHDISPLAY = new TEXTLABEL(this)
   with (this.MONTHDISPLAY)
      height = 17
      left = 24
      top = 6
      width = 99
      border = true
      colorNormal = This_ColorNormal
      alignHorizontal = 1	// Center
      alignVertical := 1	// Middle
      fontSize = 8
      fontBold = true
      text = "Janvier"
      borderStyle = 3	// None
      pageno = 0
   endwith

   this.YEARDISPLAY = new TEXTLABEL(this)
   with (this.YEARDISPLAY)
      height = 16
      left = 79
      top = 114
      width = 44
      colorNormal = This_ColorNormal
      alignHorizontal = 1	// Center
      alignVertical = 1	// Middle
      fontSize = 8
      text = "2004"
      fontBold = true
      borderStyle = 3	// None
      pageno = 0
   endwith

   this.NEXTMONTH = new TEXTLABEL(this)
   with (this.NEXTMONTH)
      onLeftMouseUp = class::NEXTMONTH_ONLEFTMOUSEUP
      height = 17
      left = 124
      width = 16
      top = 6
      fontBold = true
      text = ">"
      alignHorizontal = 1  // Center
      borderStyle = 3	// None
      colorNormal = This_ColorNormal
      pageno = 1
   endwith

   this.PREVIOUSMONTH = new TEXTLABEL(this)
   with (this.PREVIOUSMONTH)
      onLeftMouseUp = class::PREVIOUSMONTH_ONLEFTMOUSEUP
      height = 17
      left = 7
      width = 16
      fontBold = true
      text = "<"
      alignHorizontal = 1  // Center
      borderStyle = 3	// None
      top = 6
      colorNormal = This_ColorNormal
      pageno = 1
   endwith

   this.NEXTYEAR = new TEXTLABEL(this)
   with (this.NEXTYEAR)
      onLeftMouseUp = class::NEXTYEAR_ONLEFTMOUSEUP
      height = 16
      left = 124
      top = 114
      width = 16
      fontBold = true
      text = ">"
      alignHorizontal = 1  // Center
      borderStyle = 3	// None
      colorNormal = This_ColorNormal
      pageno = 1
   endwith

   this.PREVIOUSYEAR = new TEXTLABEL(this)
   with (this.PREVIOUSYEAR)
      onLeftMouseUp = class::PREVIOUSYEAR_ONLEFTMOUSEUP
      height = 16
      left = 62
      width = 16
      top = 114
      fontBold = true
      text = "<"
      alignHorizontal = 1  // Center
      borderStyle = 3	// None
      colorNormal = This_ColorNormal
      pageno = 1
   endwith

   Function setDate(dDate,cRepaint)
      /*
      Purpose: Only entry point to set default/current selected
      Date. Designed for Internal/External calls
      */

      // set defaults for missing params
      dDate      = iif(empty(dDate),date(),dDate)
      cRepaint   = iif(empty(cRepaint),'Y',cRepaint)
      this.Today = Date()   // Today's date
      this.date  = dDate    // Selected date
      /*
      this.CalDate  = ctod( str(month(this.date),2)+;
         '/01/'+str(year(this.date),4))  // Used internally to
                                          // navigate calendar
      fixed by Rick as follows
      */
      this.CalDate  = new Date( year(this.date), month(this.date)-1, 1 )
      // Used internally to navigate calendar
      // GKL The if conditional was removed because old
      // selections were not being removed in some cases
      *if cRepaint = 'Y'
         this.DrawMonth()                 // Draw calendar
      *endif
      return

   Function GetDate()
      // Purpose: returns calendar's selected date
      return this.date

   Function calendar_OnOpen
      // Container's OnOpen
      // Purpose: Ensures default date is set
      if this.language == "French"
         this.DaysDisplay.text = "  D    L     M    M    J    V     S"
         this.MonthDisplay.text = "Janvier"
      elseif this.language = "German"
         this.DaysDisplay.text = "  S    M     D    M    D    F     S"
         this.MonthDisplay.text = "Januar"
      elseif this.language = "Spanish"
         this.DaysDisplay.text = "  S    M     D    M    D    F     S"
         this.MonthDisplay.text = "Enero"
      elseif this.language = "Italian"
         this.DaysDisplay.text = "  D    L     M    M    G    V     S"
         this.MonthDisplay.text = "Gennaio"
      endif
      if type('this.CalDate') # 'D'
         this.CalDate = new date()
      endif
      this.setDate(this.date)
      * this.setDate(Date())  // JPM
      return

   Function drawMonth
      /*
      Purpose: Draws the selected month on the calendar.
      Param: this = reference to container. Make sure that
                    all calls to this method or to setDate() are
                    scoped to the container
      */
      local nMonth
      nMonth = Month(this.CalDate)
      if this.language     == "English"
         this.monthdisplay.text = cMonth(this.CalDate)
      elseif this.language == "French"
         this.monthdisplay.text = this.Translate_to_French(nMonth)
      elseif this.language == "German"
         this.monthdisplay.text = this.Translate_to_German(nMonth)
      elseif this.language == "Spanish"
         this.monthdisplay.text = this.Translate_to_Spanish(nMonth)
      elseif this.language == "Italian"
         this.monthdisplay.text = this.Translate_to_Italian(nMonth)
      endif
      this.YearDisplay.text  =  str(year(this.CalDate),4)

      // Get the last day of the month
      cMonth = str(month(this.CalDate),2)
      if at(cMonth,' 9 4 611') > 0       // if 30 day month
         nLastDay = 30
      elseif cMonth = ' 2'               // if February, do leap year
         nLastDay = iif(this.CalDate.Year%4 = 0, 29,28)
      else
         nLastDay = 31                  // Otherwise, 31
      endif

      for n = 1 to dow(this.CalDate)    // draw empty starting days
         oTextLabel = 'this.TextLabel' + ltrim(str(n,2))
         oTextLabel = &oTextLabel.
         oTextLabel.colorNormal = DEFAULTCOLOR
         oTextLabel.text = ''
      next

      // Make sure holiday list is current ...
      class::LoadHolidays()

      nDay = 1
      for n = dow(this.CalDate) to 37    // draw days of the month
         oTextLabel = 'this.TextLabel' + ltrim(str(n,2))
         oTextLabel = &oTextLabel.
         oTextLabel.colorNormal = DEFAULTCOLOR
         oTextLabel.borderStyle = 3  // None
         if nDay > nLastDay
            oTextLabel.text = ' '
         else
            oTextLabel.text = str(nDay,2)
            // GKL code modified to bridge Mayer/Martel nomenclature
            dTestDate = new date(year(this.date), month(this.date)-1,nDay)

            // Make it a real date
            dTestDate = ctod( left( dTestDate.toLocaleString(), ;
                        iif( set("CENTURY") == "ON", 10, 8 ) ) )


				if class::isHoliday( dTestDate )

               // GKL code modified to bridge Mayer/Martel nomenclature
               oTextLabel.SpeedTip := this.cSpeedTip
               oTextLabel.colorNormal := HOLIDAYCOLOR
            endif 


         endif
         nDay ++
      endfor
      // GKL Use this location to let selection color trump today color
      class::highlightToday()

      // If selected date falls on this calendar page
      // change color and redisplay
      if month(this.CalDate) = month(this.date) and ;
         year(this.CalDate)  = year(this.Date)
         n = dow(this.CalDate) + day(this.Date) -1
         cButton             = 'this.TextLabel' + ltrim(str(n,2)) + '.colorNormal'
         &cButton.          := This_ColorNormal
      endif
      // GKL Use this location to let today color trump select color
      *class::highlightToday()
      return

   Function Translate_to_French(nMonth)
      local c
      do case
         case nMonth = 1
            c = "Janvier"
         case nMonth = 2
            c = "Fvrier"
         case nMonth = 3
            c = "Mars"
         case nMonth = 4
            c = "Avril"
         case nMonth = 5
            c = "Mai"
         case nMonth = 6
            c = "Juin"
         case nMonth = 7
            c = "Juillet"
         case nMonth = 8
            c = "Aot"
         case nMonth = 9
            c = "Septembre"
         case nMonth = 10
            c = "Octobre"
         case nMonth = 11
            c = "Novembre"
         case nMonth = 12
            c = "Dcembre"
       endcase
       return(c)

   Function Translate_to_German(nMonth)
      local c
      do case
         case nMonth = 1
            c = "Januar"
         case nMonth = 2
            c = "Februar"
         case nMonth = 3
            c = "Mrz"
         case nMonth = 4
            c = "April"
         case nMonth = 5
            c = "Mai"
         case nMonth = 6
            c = "Juni"
         case nMonth = 7
            c = "Juli"
         case nMonth = 8
            c = "August"
         case nMonth = 9
            c = "September"
         case nMonth = 10
            c = "Oktober"
         case nMonth = 11
            c = "November"
         case nMonth = 12
            c = "Dezember"
       endcase
       return(c)

   Function Translate_to_Spanish(nMonth)
      local c
      do case
         case nMonth = 1
            c = "Enero"
         case nMonth = 2
            c = "Febrero"
         case nMonth = 3
            c = "Marzo"
         case nMonth = 4
            c = "Abril"
         case nMonth = 5
            c = "Mayo"
         case nMonth = 6
            c = "Junio"
         case nMonth = 7
            c = "Julio"
         case nMonth = 8
            c = "Agosto"
         case nMonth = 9
            c = "Setiembre"
         case nMonth = 10
            c = "Octubre"
         case nMonth = 11
            c = "Noviembre"
         case nMonth = 12
            c = "Diciembre"
       endcase
       return(c)

   Function Translate_to_Italian(nMonth)
      local c
      do case
         case nMonth = 1
            c = "Gennaio"
         case nMonth = 2
            c = "Febbraio"
         case nMonth = 3
            c = "Marzo"
         case nMonth = 4
            c = "Aprile"
         case nMonth = 5
            c = "Maggio"
         case nMonth = 6
            c = "Giugno"
         case nMonth = 7
            c = "Luglio"
         case nMonth = 8
            c = "Agosto"
         case nMonth = 9
            c = "Settembre"
         case nMonth = 10
            c = "Ottobre"
         case nMonth = 11
            c = "Novembre"
         case nMonth = 12
            c = "Dicembre"
       endcase
       return(c)

   Function NextMonth_onLeftMouseUp
      // Increments calendar months
      nNewMonth = month(this.parent.CalDate)
      nNewMonth ++
      if nNewMonth > 12
         nNewMonth = 1
         nNewYear  = year(this.parent.CalDate)+1
      else
         nNewYear  = year(this.parent.CalDate)
      endif
      this.parent.date    = new Date( nNewYear, nNewMonth-1, day(this.parent.date) )
      this.parent.CalDate = new Date( nNewYear, nNewMonth-1, 1 )
      this.parent.drawmonth()
      return

   Function PreviousMonth_onLeftMouseUp
      // Decrements calendar months
      nNewMonth = month(this.parent.CalDate)
      nNewMonth --
      if nNewMonth < 1
          nNewMonth = 12
          nNewYear = year(this.parent.CalDate)-1
      else
          nNewYear = year(this.parent.CalDate)
      endif
      this.parent.date    = new Date( nNewYear, nNewMonth-1, day(this.parent.date) )
      this.parent.CalDate = new Date( nNewYear, nNewMonth-1, 1 )
      this.parent.drawmonth()
      return

   Function NextYear_onLeftMouseUp
      // Increments calendar years
      dDate = this.parent.Date
      this.parent.date    = new Date( year(dDate)+1, month(dDate)-1, day(this.Parent.Date) )
      this.parent.calDate = new Date( year(dDate)+1, month(dDate)-1, 1 )
      this.parent.drawmonth()
      return

   Function PreviousYear_onLeftMouseUp
      // Decrements calendar Year
      dDate = this.parent.Date
      this.parent.date    = new Date( year(dDate)-1, month(dDate)-1, day(this.Parent.Date) )
      this.parent.calDate = new Date( year(dDate)-1, month(dDate)-1, 1 )
      this.parent.drawmonth()
      return

   Function highlightToday()
      // Marks "today" button in red/silver
      if month(this.Date) = month(date()) and ;
         year(this.Date)  = year(date())
         n = dow(this.CalDate) + day(Date()) - 1
/*  GKL code below replaced 
         cButton          = 'this.TextLabel' + ltrim(str(n,2)) + '.borderStyle'
         &cButton         = 4  // Single
*/ 
			cButton          = 'this.TextLabel' + ltrim(str(n,2)) + '.colorNormal '
			&cButton.		 := TODAYCOLOR
         cThisLabel       = 'this.TextLabel'+ltrim(str(n,2))+'.speedtip'
         &cThisLabel.     = 'Today'
      endif
      return

   Function onClick
      // empty shell to be overruled by the Calendar's form
      return

   // GKL Procedure changed to function
   Function LoadHolidays
      // Creates/re-creates array used to determine
      // a holiday ... NOTE: It adds an array
      // to the form ...
      local nYear, oRef, nHoliday, d
      local cMarkSet
      private cCmd, cDateSet
      // GKL bridges Mayer/Martel nomenclature
      this.CurrentDate = this.date 

      // Have to determine if being called from container
      // or from a button call ... 
      if "CALENDAR" $ this.className
         nYear = year( this.currentDate )
         oRef = this
      else
         nYear = year( this.parent.currentDate )
         oRef = this.parent
      endif

      // Create array
      oRef.Holidays = new Array(1,2) // 1 row, 2 columns
      
      // Load holidays using holidaydate method below:
      // floating holidays (most of these are US Holidays --
      // *********************************************
      // you may wish to modify if not in the US ...):
      // *********************************************
      // President's Day
      i = 0
      i++
      oRef.Holidays[i,1] = oRef.Holiday.HolidayDate( nYear, "P" )
      oRef.Holidays[i,2] = "President's Day"
      // Daylight Savings time -- always a Sunday
      // Memorial Day
      i++
      oRef.Holidays.grow(1)
      oRef.Holidays[i,1] = oRef.Holiday.HolidayDate( nYear, "M" )
      oRef.Holidays[i,2] = "Memorial Day"
      // Labor Day
      i++
      oRef.Holidays.grow(1)
      oRef.Holidays[i,1] = oRef.Holiday.HolidayDate( nYear, "L" )
      oRef.Holidays[i,2] = "Labor Day"
      // Columbus Day
      i++
      oRef.Holidays.grow(1)
      oRef.Holidays[i,1] = oRef.Holiday.HolidayDate( nYear, "C" )
      oRef.Holidays[i,2] = "Columbus Day"
      // Standard Time -- always a Sunday
      i++
      oRef.Holidays.grow(1)
      oRef.Holidays[i,1] = oRef.Holiday.HolidayDate( nYear, "S" )
      oRef.Holidays[i,2] = "Standard Time"
      // Daylight Savings Time
      i++
      oRef.Holidays.grow(1)
      oRef.Holidays[i,1] = oRef.Holiday.HolidayDate( nYear, "D" )
      oRef.Holidays[i,2] = "Daylight Savings Time"
      // Election Day
      i++
      oRef.Holidays.grow(1)
      oRef.Holidays[i,1] = oRef.Holiday.HolidayDate( nYear, "E" )
      oRef.Holidays[i,2] = "Election Day"
      // Thanksgiving
      i++
      oRef.Holidays.grow(1)
      oRef.Holidays[i,1] = oRef.Holiday.HolidayDate( nYear, "T" )
      oRef.Holidays[i,2] = "Thanksgiving"
      // Advent -- always a Sunday
      i++
      oRef.Holidays.grow(1)
      oRef.Holidays[i,1] = oRef.Holiday.HolidayDate( nYear, "A" )
      oRef.Holidays[i,2] = "Advent"

      // Easter requires special code ...:
      i++
      oRef.Holidays.grow(1)
      oRef.Holidays[i,1] = oRef.Holiday.EasterDay( nYear )
      oRef.Holidays[i,2] = "Easter"

      // fixed dates:
      // deal with date format, so we don't
      // mess up international users ...
      cDateSet = set("DATE")
      cMarkSet = set("MARK")
      set date MDY

      // New Year's Day
      d = ctod( "01/01/"+nYear )
      i++
      oRef.Holidays.grow(1)
      oRef.Holidays[i,1] = d
      oRef.Holidays[i,2] = "New Year's Day"
      // Lincoln's Birthday
      d = ctod( "02/12/"+nYear)
      i++
      oRef.Holidays.grow(1)
      oRef.Holidays[i,1] = d
      oRef.Holidays[i,2] = "Lincoln's Birthday"
      // Valentine's Day
      d = ctod( "02/14/"+nYear )
      i++
      oRef.Holidays.grow(1)
      oRef.Holidays[i,1] = d
      oRef.Holidays[i,2] = "Valentine's Day"
      // Washington's Birthday
      d = ctod( "02/22/"+nYear)
      i++
      oRef.Holidays.grow(1)
      oRef.Holidays[i,1] = d
      oRef.Holidays[i,2] = "Washington's Birthday"
      // St. Patrick's Day
      d = ctod( "03/17/"+nYear)
      i++
      oRef.Holidays.grow(1)
      oRef.Holidays[i,1] = d
      oRef.Holidays[i,2] = "St. Patrick's Day"
      // April Fool's Day
      d = ctod( "04/01/"+nYear)
      i++
      oRef.Holidays.grow(1)
      oRef.Holidays[i,1] = d
      oRef.Holidays[i,2] = "April Fool's Day"
      // Fourth of July
      d = ctod( "07/04/"+nYear )
      i++
      oRef.Holidays.grow(1)
      oRef.Holidays[i,1] = d
      oRef.Holidays[i,2] = "Independance Day"
      // Halloween (All Hallows Eve)
      d = ctod( "10/31/"+nYear )
      i++
      oRef.Holidays.grow(1)
      oRef.Holidays[i,1] = d
      oRef.Holidays[i,2] = "Halloween"
      // Christmas
      d = ctod( "12/25/"+nYear )
      i++
      oRef.Holidays.grow(1)
      oRef.Holidays[i,1] = d
      oRef.Holidays[i,2] = "Christmas"
      // New Year's Eve
      d = ctod( "12/31/"+nYear )
      i++
      oRef.Holidays.grow(1)
      oRef.Holidays[i,1] = d
      oRef.Holidays[i,2] = "New Year's Eve"

      // You can add more here if you wish ...
      // make sure you add a row to the array, 
      // and add the name of the holiday or memorable
      // date ... Basically, copy the code as shown
      // above, and make appropriate changes

      // reset date format to whatever it was
      // before we changed to American
      set date &cDateSet.
      cCmd = "set mark to '"+cMarkSet+"'"
      &cCmd.

      // one last thing:
      oRef.Holidays.sort(1) // sort on first column

		return

   Function IsHoliday
      parameter dCheckIt
      // this routine reads the array "this.Holidays"
      // and checks the date to see if the date in
      // dCheck parameter is in the list, and returns
      // a logical value ... 
      if "CALENDAR" $ upper( this.className )
         oRef = this
      else
         oRef = this.parent
      endif
      lHoliday = false
      oRef.cSpeedTip = ""
      for i = 1 to oRef.Holidays.size/2
          if oRef.Holidays[ i, 1 ] == dCheckIt
             lHoliday := true
             oRef.cSpeedTip = oRef.Holidays[ i, 2 ]
          endif
      next
   return ( lHoliday )

Endclass

Class TextDate(formObj) of TextLabel(formObj) // Custom
// Calbutton: Custom TextLabel class
// Purpose: contains method that fires when date is selected
   with (this)
      height = 14
      left = 25
      onLeftMouseUp = class::onLeftMouseUp
      top = 130
      width = 18
      text = " "
      fontSize = 8
      pageno = 1
      borderStyle = 3 // None
      alignHorizontal = 1	// Center
      alignVertical := 1	// Middle
      colorNormal = DEFAULTCOLOR
   endwith

   Function onLeftMouseUp
      // Purpose: Resets date when button pushed
      if empty(this.text)
         return null
      endif
      // build a date from this button text and
      // current month/year as displayed
      cDispMonth = this.parent.monthdisplay.text
      nPos = 0
      if this.parent.language == "English"
         do
            nPos++
         until cDispMonth = cMonth( new Date( year(date()), nPos-1, 1) )
      elseif this.parent.language == "French"
         do
            nPos++
         until cDispMonth = this.parent.Translate_to_French(Month( new Date( year(date()), nPos-1, 1) ))
      elseif this.parent.language == "German"
         do
            nPos++
         until cDispMonth = this.parent.Translate_to_German(Month( new Date( year(date()), nPos-1, 1) ))
      elseif this.parent.language == "Spanish"
         do
            nPos++
         until cDispMonth = this.parent.Translate_to_Spanish(Month( new Date( year(date()), nPos-1, 1) ))
      elseif this.parent.language == "Italian"
         do
            nPos++
         until cDispMonth = this.parent.Translate_to_Italian(Month( new Date( year(date()), nPos-1, 1) ))
      endif
      // turn off this color if it shows onscreen
      this.parent.setDate( ( new Date( val(this.parent.YearDisplay.text),;
                           nPos-1, val(this.text))), 'N' )
      // check all the buttons to see if any are highlighted as
      // "selected" date. If so, ditch the highlight
      oStart = this.parent.TextLabel1
      oCurrent = oStart
      do
         if upper(oCurrent.classname) = 'TEXTDATE'
            if oCurrent.colornormal = This_ColorNormal
               oCurrent.colornormal = DEFAULTCOLOR
            endif
            oCurrent.borderStyle = 3  // None
         endif
         oCurrent = oCurrent.before
      until oCurrent = oStart
      // highlight "today" if appropriate
      this.parent.highlightToday()
      // highlight this button as current date
      this.colornormal := This_ColorNormal
      /*
      This class now supports the following:
      1) a passed function pointer for callback
      2) a Current "owner" object for callback
      3) a Current "field" for auto update
      */
      // current owners callback
      // requires that it also have a "setdate" method
      private n
      for n =1 to this.parent.aCurrentOwners.size
         if type('this.parent.aCurrentOwners[n]') = 'O' and ;
            type('this.parent.aCurrentOwners[n].setDate') = 'FP' // if setdate exists
            this.parent.aCurrentOwners[n].setDate(this.parent.Date)
         endif
      next
      ///// current field
      for n = 1 to this.parent.aCurrentFields.size
         if type('this.parent.aCurrentFields[n].value') = 'D' // if it's a date field
            this.parent.aCurrentFields[n].value = this.parent.Date
         endif
      next
      this.parent.onClick()
      return null

Endclass
