/*
     SKIPLABL.PRG
     Programmer: Ken Mayer
     Date......: 02/20/1998

     This program is designed to skip 'n' labels on a sheet
     to print at the next streamFrame. This is simply an attempt
     to save labels -- there are many situations where the last
     time a set of labels were run, only some of the labels
     were used on the last sheet. 

     Rather than tossing that last sheet of labels in the trash,
     it may be desireable to start at the next available label
     on the first page of labels, so that you can use that 
     partial sheet again and not waste it.

     The suggestion for this came from Rainald Taesler on the
     VdBASE forum on Compuserve. Code by Ken Mayer.

     NOTE: This requires a parameter for the label file name ...
  
     DO SKIPLABL WITH "labelname.lab", nLabels  or   
     DO SKIPLABL WITH "labelname", nLabels

     In either case, 'nLabels' is the number of labels to _skip_
     (not the label to start at, but to skip ...)

     **************************************************************
     WARNING: Many label manufacturers warn against reusing labels
     in laser printers -- there is a reason for this, and the 
     author is not responsible if you end up with labels peeling
     off the sheet and sticking to the drum of your printer ...
     **************************************************************

*/


// deal with missing label name:
if argCount() = 0
   msgbox( "Need a label name to work with ...", ;
           "Can't function", 16 )
   return
endif

// deal with missing skip value:
if argCount() = 1
   msgbox( "Need to know how many labels to skip ...",;
           "Can't continue!", 16 )
   return
endif

// store number of labels to skip:
nSkipLabels = argVector(2)

// check to see if report exists:
cFile = argVector(1)
if upper( right( cFile, 4 ) ) # ".LAB"
   cFile = cFile + ".LAB"
endif

if not file( cFile )
   msgbox( "Label file: "+cFile+" does not exist!", ;
           "Can't function", 16 )
   return
endif

// Ok. At this point, the .LAB file exists, let's get
// the className from the constructor code:
f = new File()
f.open( cFile )
do while not f.eof()
   cLine = upper( f.Gets() )
   if "CLASS" $ upper( cLine ) and;
      "REPORT" $ upper( cLine ) /* hopefully this is the correct
                                   line -- i.e. the word 'class' doesn't
                                   appear in comments at the beginning
                                   of the source for some reason ... */
      nFirstSpace = at( " ", cLine )
      nNextSpace  = at( " ", cLine, 2 )
      cLabelName = substr( cLine, nFirstSpace+1, ( nNextSpace-nFirstSpace ) )
      exit
   endif
   
enddo
f.close()

// Next, create an instance of the report ...:
set procedure to &cFile. additive
rMyLab = new &cLabelName.()
cFile2 = cFile.left( cFile.indexOf( "." ) ) // no .LAB

/*
   For this to work, we're going to have to have to use new
   features for Visual dBASE 7.01: streamSource.beginNewFrame() 
   method, the streamFrame.canRender() event,
   and possibly the band.streamframe property

   This _must_ assume that the default names for pageTemplate and
   streamFrames are used -- if not, then this will not work!
*/
p = rMyLab.pageTemplate1 // shorten references

// Loop through the streamFrames ...
for i = 1 to nSkipLabels
    cEvent = "p.streamFrame"+i+".canRender"
    // the event must be set ONLY for the first page -- otherwise
    // we will skip this streamFrame for all pages ...:
    &cEvent. = {; iif( this.parent.parent.reportPage = 1, this.parent.parent.streamSource1.beginNewFrame(), null ) }
next

rMyLab.render()

// a bit of cleanup:
close procedure &cFile.
rMyLab.release()

/* ------------------------------------------------------------
   End of Program: SKIPLABL.PRG
   ------------------------------------------------------------ */


