/*
Class:   Font
Author:  Daniel Howard (sproket@total.net)
History: 
   1.0 Mar 27, 1999


Purpose:
	This class hold/handles font information.

Properties:
   FontName 	
   FontSize 
   FontBold
   FontItalic
   FontStrikeout
   FontUnderline
   FontFamily 		// Not used 

Methods:

	DefineFrom	-  Takes a font string or an object as a parameter and
   					initializes itself from that information.
		eg:
      cFont = GetFont()
      oFont.dDefineFrom(cFont)

      or

      oFont.DefineFrom(form.EntryField1)

	AsString	-		Returns the string version of the font object.

		eg:
		   Arial,10,B,Swiss

	SetObjectFont-	This method set's an object's font properties to this 
						classes properties. 

		eg:
	   oFont.DefineFrom(form.EntryField1)                  
      oFont.SetObjectFont(form.PushButton2)


Notes:


This software is donated to the public domain. You may modify 
this code in anyway you wish however the author reserves the 
right to whatever official version is published. If you have 
further ideas/bug reports/additions then contact the author and 
your name will be added to the contributors if your idea is 
implemented.

The author accepts no responsibility for the use or consequences 
of the use of this software.
*/


Class Font
protect StrExtract,CharCount


   this.FontName = ""
   this.FontSize = 0
   this.FontBold = false
   this.FontItalic = false
   this.FontStrikeout = false
   this.FontUnderline = false
   this.FontFamily = "" // Not used 


   // Takes a font string or an object and defines this object's
   // properties from it.
   Function DefineFrom
   parameter xFont

      if type("xFont") == "O"        
         this.FontName := xFont.FontName
         this.FontSize := xFont.FontSize
         this.FontItalic := xFont.FontItalic
         this.FontStrikeout := xFont.FontStrikeout
         this.FontUnderline := xFont.FontUnderline
         this.FontBold := xFont.FontBold
      else        // String
         this.FontName = Class::StrExtract(1,xFont,",")
         this.FontSize = Val(Class::StrExtract(2,xFont,","))
         if Class::CharCount(xFont,",") >= 2
            this.FontBold = "B" $ upper(Class::StrExtract(3,xFont,","))
            this.FontItalic = "I" $ upper(Class::StrExtract(3,xFont,","))
            this.FontStrikeout = "S" $ upper(Class::StrExtract(3,xFont,","))
            this.FontUnderline = "U" $ upper(Class::StrExtract(3,xFont,","))
         else
            this.FontBold = false
            this.FontItalic = false
            this.FontStrikeout = false
            this.FontUnderline = false
         endif
      endif              
   return


   // This returns the current object's properties as a string
   // in the same format that the GetFont() would return.
   Function AsString
   local cStyle
   local cFont

      cStyle = ""
      cStyle += iif(this.FontBold,"B","")
      cStyle += iif(this.FontItalic,"I","")
      cStyle += iif(this.FontUnderline,"U","")
      cStyle += iif(this.FontStrikeout,"S","")

      if empty(cStyle)
         cFont = this.FontName+","+str(this.FontSize,2,0)
      else
         cFont = this.FontName+","+str(this.FontSize,2,0)+","+cStyle
      endif
   return ( cFont )

   // This set's a passed object's font information from this object's
   // font information.
   Function SetObjectFont(o)
      o.FontName = this.FontName
      o.FontSize = this.FontSize
      o.FontBold = this.FontBold
      o.FontItalic = this.FontItalic
      o.FontStrikeout = this.FontStrikeout
      o.FontUnderline = this.FontUnderline
   return

   function StrExtract(nItem,cStr,cToken)
   local p1,p2,j
   local cItem
   local cSubStr
      if valtype(cStr) <> "C"
         return ""
      endif
   
      p1 = 1
      p2 = 0
      cSubStr = cStr
      cItem  = ""
      j = 1
      do while .not. empty(cSubStr)
         pos = at(cToken,cSubStr)-1
         if pos <= 0
            pos = len(cSubStr)
         endif
         cItem = left(cSubstr,pos)
         if j = nItem
            exit
         endif
         if pos < len(cSubStr) - 1
            cSubStr = substr(cSubStr,pos+2,len(cSubStr))
            if cToken == " "
               cSubStr = ltrim(cSubStr)
            endif
         else
            cSubStr = ""
            cItem   = ""
         endif
         j = j + 1
      enddo
   return (cItem)

   function CharCount(cStr,cChar)
   local j
   local count
   local nLen
   
      nLen  = len(cChar)
      count = 0
      for j = 1 to len(cStr)
         if cChar = substr(cStr,j,nLen)
            count = count + 1
         endif
      next j
   return ( count )   


endclass
