/*
   kmScheduler.h
   This is a set of definitions of strings used in the forms of
   the Calendar/Scheduler created by Ken Mayer. The idea is to
   place all character strings that are displayed for the user
   in this file as defined constants (using the preprocessor
   command: #DEFINE). In the code for the four different forms,
   anywhere that a character string is displayed, it should be
   replaced by the appropriate constant in the header file.
   This will have the effect of making it possible to have
   folk who know other languages as well as English translate
   the strings, and make this more non-English capable. 

   When  using these, each of the following four forms will need
   to have the command:
       #INCLUDE <kmScheduler_English.h>
   changed to use the appropriate header file, such as:
       #INCLUDE <kmScheduler_German.h>

   The forms are:
       kmScheduler.wfm
       kmSchedEvent.wfm
       km24Time.wfm
       kmSmallCal.wfm

   For those doing the translation, if any of the character strings
   require the use of upper ANSI characters, such as u with an umlaut
   (just an example), rather than typing that in, it may be a good
   idea to use chr( 252 ), as in: "abcde"+chr(252)+"test".
*/ 

// ------------------------------------------------------------------
// For the main calendar form: kmScheduler.wfm
// ------------------------------------------------------------------
// Form's text property:
#DEFINE SCHEDFORMTEXT             "Kalender/Terminplaner"

// UI -- User Interface -- speedtips for controls:
#DEFINE MONTHBUTTONTEXT           "Monat"
#DEFINE NEXTMONTHBUTTONSPEEDTIP   "Nchster Monat"
#DEFINE PREVMONTHBUTTONSPEEDTIP   "Vorheriger Monat"
#DEFINE YEARBUTTONTEXT            "Jahr"
#DEFINE NEXTYEARBUTTONSPEEDTIP    "Nchstes Jahr"
#DEFINE PREVYEARBUTTONSPEEDTIP    "Vorheriges Jahr"
#DEFINE TODAYBUTTONTEXT           "Heute"
#DEFINE TODAYBUTTONSPEEDTIP       "Gehe zum heutigen Tag"
#DEFINE WEEKVIEWBUTTONTEXT        "Wochennansicht"
#DEFINE WEEKVIEWBUTTONSPEEDTIP    "Zur Wochenansicht wechseln"
#DEFINE WEEKBUTTONTEXT            "Woche"
#DEFINE NEXTWEEKBUTTONSPEEDTIP    "Nchste Woche"
#DEFINE PREVWEEKBUTTONSPEEDTIP    "Vorherige Woche"
#DEFINE MONTHVIEWBUTTONTEXT       "Monatsansicht"
#DEFINE MONTHVIEWBUTTONSPEEDTIP   "Zur Monatsansicht wechseln"
#DEFINE MOREBUTTONUPSPEEDTIP      "Vorangehende fnf Eintrge ansehen"
#DEFINE MOREBUTTONDOWNSPEEDTIP    "Nchste fnf Eintrge ansehen"

// canOpen event:
#DEFINE OPENFORMERROR             "Kann das Formular nicht ffnen!"
#DEFINE DATEEXOPENERROR           ":dUFLP:DateEx.cc kann nicht geffnet werden!"
#DEFINE HOLIDAYOPENERROR          ":dUFLP:Holiday.cc kann nicht geffnet werden!"
#DEFINE BDEALIASOPENERROR         ":dUFLP:BDEAlias.cc kann nicht geffnet werden!"
// note the trailing space before the quote -- keep it!
#DEFINE CREATEALIASERROR          "Fehler beim Erstellen von BDE Alias: "
#DEFINE CREATETABLEERROR          "Fehler bein Erstellen von Tabelle: "
#DEFINE TABLELEVELERROR           "Table Level 7 erforderlich -- bitte im BDE Administrator ndern!"
#DEFINE CREATEINDEXESERROR        "Fehler beim Erstellen der Indizes zu Tabelle: "
// Holiday names:
#DEFINE ADVENTNAME                "Advent"
#DEFINE EASTERNAME                "Ostern"
#DEFINE NEWYEARNAME               "Neujahrstag"
#DEFINE VALENTINENAME             "Valentinstag"
#DEFINE STPATRICKNAME             "St. Patrick's Day"
#DEFINE APRILFOOLNAME             "1. April"
#DEFINE HALLOWEENNAME             "Halloween"
#DEFINE CHRISTMASNAME             "Weihnachten"
#DEFINE NEWYEAREVENAME            "Silvester"


// ------------------------------------------------------------------
// kmSchedEvent.wfm
// ------------------------------------------------------------------
// UI Interface:
// Form text:
#DEFINE SCHEDEVENTFORMTEXT        "Details zu Termin(en)"
// Tabbox1 dataSource -- don't translate the word "ARRAY":
#DEFINE TABBOXDATASOURCE          [ARRAY {"Aktueller Termin","Termine des Tages"}]
// Title text:
#DEFINE TITLETEXTEVENTS           "Termine"
// Grid column headings:
#DEFINE GRIDCOLUMN1HEADING        "Zeit"
#DEFINE GRIDCOLUMN2HEADING        "Termin"
#DEFINE GRIDCOLUMN3HEADING        "Wiederholung"
// Text/TextLabel controls on form:
#DEFINE EVENTTITLETEXT            "Termin:"
#DEFINE DETAILTEXT                "Details:"
#DEFINE LOCATIONTEXT              "Ort:"
#DEFINE DATETEXT                  "Datum:"
#DEFINE STARTTIMETEXT             "Anfangszeit:"
#DEFINE ENDTIMETEXT               "Ende:"
#DEFINE REPEATTYPETEXT            "Wiederholung:"
#DEFINE REPEATEVENTSTEXT          "Wiederkehrende Termine"
#DEFINE EVENTNOTREPEATTEXT        "Dieser Termin wiederholt sich nicht"
#DEFINE STATUSTEXT                "Status:"
// Note the HTML formatting here -- leave those, but move as appropriate:
#DEFINE INFOTEXT                  "<b>Datum</b> ist erforderlich, Standardwert "+;
                                  "ist das Datum des Termins. "+;
                                  "<b>Anfangszeit</b> und <b>Ende</b> "+;
                                  "sind erforderlich, wenn es sich nicht um einen "+;
                                  "ganztgigen Termin handelt. "+;
                                  "Fr alle Zeitangaben das 24-Stunden-Format "+;
                                  "verwenden."
// Checkbox:
#DEFINE ALLDAYCHECKTEXT           "Ganzer Tag?"
// Buttons:
#DEFINE CHANGEDATEBUTTONSPEEDTIP  "Datum ndern"
#DEFINE STARTTIMESPEEDTIP         "Anfangszeit ndern"
#DEFINE ENDTIMESPEEDTIP           "Ende ndern"
// Comboboxes:
#DEFINE REPEATTYPE1               "Keine"
#DEFINE REPEATTYPE2               "Tag"
#DEFINE REPEATTYPE3               "Woche"
#DEFINE REPEATTYPE4               "Monat"
#DEFINE REPEATTYPE5               "Jahr"
#DEFINE STATUSCOMBO1              "Offen"
#DEFINE STATUSCOMBO2              "Erledigt"
// On 'repeat' containers:
#DEFINE EVERYTEXT                 "Alle:"
#DEFINE ENDONTEXT                 "Bis:"
#DEFINE REPEATSPINERROR           "Wertebereich von 1 bis 99"
#DEFINE ENDDATEBUTTONSPEEDTIP     "Datum ndern"
#DEFINE DAYSTEXT                  "Tage"
#DEFINE WEEKSTEXT                 "Wochen"
#DEFINE MONTHSTEXT                "Monate"
#DEFINE YEARSTEXT                 "Jahre"
#DEFINE DESCRIPTTEXT              "Der n-te eines jeden Monats"
#DEFINE REPEATONTEXT              "Wiederhole bei:"
// Delete method:
#DEFINE DELETEQUESTIONTEXT        "Mchten Sie nur den "+;
                                  "aktuellen Termin lschen (Ja), oder alle "+;
                                  "Vorkommen dieses Termins (Nein)?"
#DEFINE REPEATINGEVENTTITLE       "Regelmiger Termin"

#DEFINE UNEXPECTEDERRORMSG        "Unerwarteter Fehler: "
#DEFINE ERRORTITLE                "Fehler"
// Save Method:
#DEFINE SAVEREPEATQUESTIONTEXT    "Sollen die nderungen nur auf diesen Termin "+;
                                  "angewendet werden (Ja) oder alle Vorkommen " + ;
                                  "dieses Termins (Nein)?" 
#DEFINE CANNOTSAVETITLE           "Speichern nicht mglich!"
#DEFINE TITLEREQUIREDTEXT         "Name des Termins muss angegeben werden!"

#DEFINE EVENTREQUIREDTEXT         "Datum des Termins muss angegeben werden!"
#DEFINE STARTTIMEREQUIREDTEXT     "Anfangszeit des Termins muss angegeben werden!"
#DEFINE ENDTIMEREQUIREDTEXT       "Ende des Termins muss angegeben werden!"
#DEFINE ENDDATEREQUIREDTEXT       "Ein Endedatum ist erforderlich fr einen regelmigen Termin!"
#DEFINE DAYOFWEEKREQUIREDTEXT     "Mindestens ein Wochentag muss angegeben "+;
                                  "fr einen wchentlich wiederkehrenden Termin!"
// Week Day checkbox change:
#DEFINE WEEKDAYERROR1             "Das aktuell ausgewhlte Datum: "
#DEFINE WEEKDAYERROR2             "ist ein "
#DEFINE WEEKDAYERROR3             ", es kann nicht gelscht werden."
#DEFINE WEEKDAYERRORTITLE         "Aktion nicht erlaubt"
// TimeEntryfield class:
#DEFINE TIMEENTRYSPEEDTIP         "Zeit im 24-Stunden-Format eingeben"
#DEFINE TIMEENTRYHOURERRORTEXT    "Wert zwischen 0 und 23 eingeben"
#DEFINE TIMEENTRYHOURERRORTITLE   "Ungltige Stundenangabe"
#DEFINE TIMEENTRYMINUTEERRORTEXT  "Wert zwischen 0 und 59 eingeben"
#DEFINE TIMEENTRYMINUTEERRORTITLE "Ungltige Minutenanagabe"



// ------------------------------------------------------------------
// km24Time.wfm
// ------------------------------------------------------------------
#DEFINE TIMEFORMTEXT              "Zeit auswhlen"
#DEFINE HOURTEXT                  "Stunden"
#DEFINE MINUTETEXT                "Minuten"
#DEFINE TIMEFORMDESCRIPTIONTEXT   "Zeitangaben im 24-Stunden-Format"
#DEFINE OKBUTTONTEXT              "Ok"
#DEFINE TIMEOKBUTTONSPEEDTIP      "Ausgewhlte Uhrzeit bernehmen"
#DEFINE CANCELBUTTONTEXT          "Abbruch"
#DEFINE TIMECANCELBUTTONSPEEDTIP  "Vorherigen Wert wieder herstellen"

// ------------------------------------------------------------------
// kmSmallCal.wfm
// ------------------------------------------------------------------
#DEFINE CALFORMTEXT               "Datum auswhlen"
#DEFINE DATEOKBUTTONSPEEDTIP      "Ausgewhltes Datum bernehmen"

// ------------------------------------------------------------------
// :dUFLP:kenCal3.cc (on kmSmallCal.wfm)
// ------------------------------------------------------------------
#DEFINE KCYEARMINUS10BUTTONSPEEDTIP "10 Jahre zurck"
#DEFINE KCYEARPLUS10BUTTONSPEEDTIP  "10 Jahre weiter"
#DEFINE KCPREVYEARBUTTONSPEEDTIP    "Vorangehendes Jahr"
#DEFINE KCNEXTYEARBUTTONSPEEDTIP    "Nchstes Jahr"
#DEFINE KCMONTHMINUS3BUTTONSPEEDTIP "3 Monate zurck"
#DEFINE KCMONTHPLUS3BUTTONSPEEDTIP  "3 Monate weiter"
#DEFINE KCPREVMONTHBUTTONSPEEDTIP   "Vorangehender Monat"
#DEFINE KCNEXTMONTHBUTTONSPEEDTIP   "Nchster Monat"
#DEFINE KCTODAYBUTTONTEXT           "Heute"
#DEFINE KCTODAYBUTTONSPEEDTIP       "Gehe zum heutigen Datum"

// ------------------------------------------------------------------
// kmSchedColor.wfm
// ------------------------------------------------------------------
// This one is actually in kmScheduler.wfm
#DEFINE CHANGECOLORBUTTONSPEEDTIP     "Farbe der Buttons ndern"
// The rest are in kmSchedColor.wfm
#DEFINE COLORFORMTITLE                "Farben ndern"
#DEFINE HOLIDAYBUTTONTEXT             "Feiertag"
#DEFINE CURRENTDAYBUTTONTEXT          "Aktuell ausgewhlter Tag"
#DEFINE DEFAULTDAYBUTTONTEXT          "Standard"
#DEFINE TEXTCOLORBUTTONTEXT           "V"
#DEFINE BACKGROUNDCOLORBUTTONTEXT     "H"
#DEFINE TEXTCOLORBUTTONSPEEDTIP       "Textfarbe ndern"
#DEFINE BACKGROUNDCOLORBUTTONSPEEDTIP "Hintergrundfarbe ndern"
