/*
   kmScheduler.h
   This is a set of definitions of strings used in the forms of
   the Calendar/Scheduler created by Ken Mayer. The idea is to
   place all character strings that are displayed for the user
   in this file as defined constants (using the preprocessor
   command: #DEFINE). In the code for the four different forms,
   anywhere that a character string is displayed, it should be
   replaced by the appropriate constant in the header file.
   This will have the effect of making it possible to have
   folk who know other languages as well as English translate
   the strings, and make this more non-English capable. 

   When  using these, each of the following four forms will need
   to have the command:
       #INCLUDE <kmScheduler_English.h>
   changed to use the appropriate header file, such as:
       #INCLUDE <kmScheduler_German.h>

   The forms are:
       kmScheduler.wfm
       kmSchedEvent.wfm
       km24Time.wfm
       kmSmallCal.wfm

   For those doing the translation, if any of the character strings
   require the use of upper ANSI characters, such as u with an umlaut
   (just an example), rather than typing that in, it may be a good
   idea to use chr( 252 ), as in: "abcde"+chr(252)+"test".
*/ 

// ------------------------------------------------------------------
// For the main calendar form: kmScheduler.wfm
// ------------------------------------------------------------------
// Form's text property:
#DEFINE SCHEDFORMTEXT             "Calendrio/Agenda"

// UI -- User Interface -- speedtips for controls:
#DEFINE MONTHBUTTONTEXT           "Ms"
#DEFINE NEXTMONTHBUTTONSPEEDTIP   "Ms Seguinte"
#DEFINE PREVMONTHBUTTONSPEEDTIP   "Ms Anterior"
#DEFINE YEARBUTTONTEXT            "Ano"
#DEFINE NEXTYEARBUTTONSPEEDTIP    "Ano Seguinte"
#DEFINE PREVYEARBUTTONSPEEDTIP    "Ano Anterior"
#DEFINE TODAYBUTTONTEXT           "Hoje"
#DEFINE TODAYBUTTONSPEEDTIP       "Ir para o dia de Hoje"
#DEFINE WEEKVIEWBUTTONTEXT        "Vista Semanal"
#DEFINE WEEKVIEWBUTTONSPEEDTIP    "Mudar para Vista Semanal"
#DEFINE WEEKBUTTONTEXT            "Semana"
#DEFINE NEXTWEEKBUTTONSPEEDTIP    "Semana Seguinte"
#DEFINE PREVWEEKBUTTONSPEEDTIP    "Semana Anterior"
#DEFINE MONTHVIEWBUTTONTEXT       "Vista Mensal"
#DEFINE MONTHVIEWBUTTONSPEEDTIP   "Mudar para Vista Mensal"
#DEFINE MOREBUTTONUPSPEEDTIP      "Ver os cinco items anteriores"
#DEFINE MOREBUTTONDOWNSPEEDTIP    "Ver os cinco items seguintes"

// canOpen event:
#DEFINE OPENFORMERROR             "No posso abrir Formulario"
#DEFINE DATEEXOPENERROR           "No posso abrir :dUFLP:DateEx.cc!"
#DEFINE HOLIDAYOPENERROR          "No posso abrir :dUFLP:Holiday.cc!"
#DEFINE BDEALIASOPENERROR         "No posso abrir :dUFLP:BDEAlias.cc!"
// note the trailing space before the quote -- keep it!
#DEFINE CREATEALIASERROR          "No posso criar BDE Alias: "
#DEFINE CREATETABLEERROR          "No posso criar tabela: "
#DEFINE TABLELEVELERROR           "A Tabela tem de ser Nvel 7 - por favor faa a alterao no Administrador do BDE!"
#DEFINE CREATEINDEXESERROR        "No posso criar indices na tabela: "
// Holiday naMs:
#DEFINE ADVENTNAME                "Advento"
#DEFINE EASTERNAME                "Pscoa"
#DEFINE NEWYEARNAME               "Dia de Ano Novo"
#DEFINE VALENTINENAME             "Dia de S.Valentim"
#DEFINE STPATRICKNAME             "Dia de S. Patrcio"
#DEFINE APRILFOOLNAME             "Dia das Mentiras"
#DEFINE HALLOWEENNAME             "Dia das Bruxas"
#DEFINE CHRISTMASNAME             "Natal"
#DEFINE NEWYEAREVENAME            "Vspera de Ano Novo"


// ------------------------------------------------------------------
// kmSchedEvent.wfm
// ------------------------------------------------------------------
// UI Interface:
// Form text:
#DEFINE SCHEDEVENTFORMTEXT        "Detalhes individuais dos Compromissos"
// Tabbox1 dataSource -- don't translate the word "ARRAY":
#DEFINE TABBOXDATASOURCE          [ARRAY {"Compromisso Corrente","Compromissos do Dia"}]
// Title text:
#DEFINE TITLETEXTEVENTS           "Compromisso(s)"
// Grid column headings:
#DEFINE GRIDCOLUMN1HEADING        "Hora"
#DEFINE GRIDCOLUMN2HEADING        "Compromisso"
#DEFINE GRIDCOLUMN3HEADING        "Repetir"
// Text/TextLabel controls on form:
#DEFINE EVENTTITLETEXT            "Ttulo do Compromisso: "
#DEFINE DETAILTEXT                "Detalhe:"
#DEFINE LOCATIONTEXT              "Local:"
#DEFINE DATETEXT                  "Data:"
#DEFINE STARTTIMETEXT             "Hora de Incio:"
#DEFINE ENDTIMETEXT               "Hora de Fim:"
#DEFINE REPEATTYPETEXT            "Repetir o Tipo:"
#DEFINE REPEATEVENTSTEXT          "Repetir Compromissos"
#DEFINE EVENTNOTREPEATTEXT        "Este Compromisso No se repete"
#DEFINE STATUSTEXT                "Status:"
// Note the HTML formatting here -- leave those, but move as appropriate:
#DEFINE INFOTEXT                  "<b>Data</b>  necessria, e por predefinio  a data seleccionada para o compromisso. <b>Hora de Incio</b> e <b>Hora de Fim</b> so necessrias excepto se fr um compromisso para 'Todo o Dia'. As Horas devem estar no formato 24 horas (se depois do meio-dia, some 12 -- i.e., 1:00 da tarde so 13:00)."
// Checkbox:
#DEFINE ALLDAYCHECKTEXT           "Todo o Dia?"
// Buttons:
#DEFINE CHANGEDATEBUTTONSPEEDTIP  "Mudar Data"
#DEFINE STARTTIMESPEEDTIP         "Mudar Hora de Incio"
#DEFINE ENDTIMESPEEDTIP           "Mudar Hora de Fim"
// Comboboxes:
#DEFINE REPEATTYPE1               "Nenhum"
#DEFINE REPEATTYPE2               "Dia"
#DEFINE REPEATTYPE3               "Semana"
#DEFINE REPEATTYPE4               "Ms"
#DEFINE REPEATTYPE5               "Ano"
#DEFINE STATUSCOMBO1              "Aberto"
#DEFINE STATUSCOMBO2              "Completo"
// On 'repeat' containers:
#DEFINE EVERYTEXT                 "Todos(as)"
#DEFINE ENDONTEXT                 "Termina em:"
#DEFINE REPEATSPINERROR           "Vlido apenas de 1 a 99"
#DEFINE ENDDATEBUTTONSPEEDTIP     "Mudar Data"
#DEFINE DAYSTEXT                  "Dia(s)"
#DEFINE WEEKSTEXT                 "Semana(s)"
#DEFINE MONTHSTEXT                "Ms(es)"
#DEFINE YEARSTEXT                 "Ano(s)"
#DEFINE DESCRIPTTEXT              "No (dia) de cada Ms "
#DEFINE REPEATONTEXT              "Repetir cada:"
// Delete method:
#DEFINE DELETEQUESTIONTEXT        "Deseja apagar apenas o compromisso actual (Sim), ou todas as ocorrncias deste compromisso (No)?"
#DEFINE REPEATINGEVENTTITLE       "Compromisso recorrente"
#DEFINE UNEXPECTEDERRORMSG        "Erro inesperado: "
#DEFINE ERRORTITLE                "Erro"
// Save Method:
#DEFINE SAVEREPEATQUESTIONTEXT    "Deseja que as alteraes a este compromisso afectem apenas o compromisso actual (Sim), ou todas as ocorrncias deste compromisso (No)?"
#DEFINE TITLEREQUIREDTEXT         "Falta o Ttulo do Compromisso!"
#DEFINE CANNOTSAVETITLE           "No posso Guardar!"
#DEFINE EVENTREQUIREDTEXT         "Falta a Data do Compromisso!"
#DEFINE STARTTIMEREQUIREDTEXT     "Falta a Data de Incio do Compromisso!"
#DEFINE ENDTIMEREQUIREDTEXT       "Falta a Data de Fim do Compromisso!"
#DEFINE ENDDATEREQUIREDTEXT       " necessria uma Data de Fim para um Compromisso Recorrente!"
#DEFINE DAYOFWEEKREQUIREDTEXT     "Pelo menos um Dia da Semana tem de ser seleccionado para um Compromisso 'Semanal'!"
// Week Day checkbox change:
#DEFINE WEEKDAYERROR1             "A data actualmente seleccionada: "
#DEFINE WEEKDAYERROR2             " "
#DEFINE WEEKDAYERROR3             ",  no pode ser desseleccionada."
#DEFINE WEEKDAYERRORTITLE         "No pode fazer isso"
// TimeEntryfield class:
#DEFINE TIMEENTRYSPEEDTIP         "Introduza um valor no formato 24 horas"
#DEFINE TIMEENTRYHOURERRORTEXT    "Introduza um valor de 0 a 23"
#DEFINE TIMEENTRYHOURERRORTITLE   "Hora Invlida"
#DEFINE TIMEENTRYMINUTEERRORTEXT  "Introduza um valor de 0 a 59"
#DEFINE TIMEENTRYMINUTEERRORTITLE "Minutos Invlidos"

// ------------------------------------------------------------------
// km24Time.wfm
// ------------------------------------------------------------------
#DEFINE TIMEFORMTEXT              "Seleccione a Hora"
#DEFINE HOURTEXT                  "Hora"
#DEFINE MINUTETEXT                "Minutos"
#DEFINE TIMEFORMDESCRIPTIONTEXT   "Hora no formato 24 horas, assim para qualquer hora da tarde, some 12. Exemplo, 2:00 da tarde sero 14:00. Meia noite so '00'."
#DEFINE OKBUTTONTEXT              "Ok"
#DEFINE TIMEOKBUTTONSPEEDTIP      "Aceitar a Hora actual"
#DEFINE CANCELBUTTONTEXT          "Cancelar"
#DEFINE TIMECANCELBUTTONSPEEDTIP  "Repr valor prviamente seleccionado"

// ------------------------------------------------------------------
// kmSmallCal.wfm
// ------------------------------------------------------------------
#DEFINE CALFORMTEXT               "Seleccionar Data"
#DEFINE DATEOKBUTTONSPEEDTIP      "Aceitar data actual"

// ------------------------------------------------------------------
// :dUFLP:kenCal3.cc (on kmSmallCal.wfm)
// ------------------------------------------------------------------
#DEFINE KCYEARMINUS10BUTTONSPEEDTIP "10 Anos Antes"
#DEFINE KCYEARPLUS10BUTTONSPEEDTIP  "10 Anos Depois"
#DEFINE KCPREVYEARBUTTONSPEEDTIP    "Ano Anterior"
#DEFINE KCNEXTYEARBUTTONSPEEDTIP    "Ano Seguinte"
#DEFINE KCMONTHMINUS3BUTTONSPEEDTIP "3 Mses Antes"
#DEFINE KCMONTHPLUS3BUTTONSPEEDTIP  "3 Mses Depois"
#DEFINE KCPREVMONTHBUTTONSPEEDTIP   "Ms Anterior"
#DEFINE KCNEXTMONTHBUTTONSPEEDTIP   "Ms Seguinte"
#DEFINE KCTODAYBUTTONTEXT           "Hoje"
#DEFINE KCTODAYBUTTONSPEEDTIP       "Ir para Hoje"

// ------------------------------------------------------------------
// kmSchedColor.wfm
// ------------------------------------------------------------------
// This one is actually in kmScheduler.wfm
#DEFINE CHANGECOLORBUTTONSPEEDTIP     "Mudar a cr dos botes"
// The rest are in kmSchedColor.wfm
#DEFINE COLORFORMTITLE                "Mudar as cores"
#DEFINE HOLIDAYBUTTONTEXT             "Feriado"
#DEFINE CURRENTDAYBUTTONTEXT          "Dia Seleccionado"
#DEFINE DEFAULTDAYBUTTONTEXT          "Predefinio"
#DEFINE TEXTCOLORBUTTONTEXT           "T"
#DEFINE BACKGROUNDCOLORBUTTONTEXT     "F"
#DEFINE TEXTCOLORBUTTONSPEEDTIP       "Mudar a cr do textot"
#DEFINE BACKGROUNDCOLORBUTTONSPEEDTIP "Mudar a cr do fundo"

