/*
   MAKEINDEXLIB.PRG - Version 1.8

   First Version: September 29, 1998

   Second Version: September 30, 1998
                   Removed _app.Flag.Close() call in several places
                   (I use it, but maybe you don't; therefore it shouldn't be there)

   Third Version:  October 1, 1998
                   Added TRY/CATCH around ERASE in case EXCLUSIVE failed
                   (based on an idea from Gary White - thanks)
                   Changed EXIT text to CONTINUE

   Fourth Version: October 2, 1998
                   Changed the outputted code to be about %30 smaller by removing
                   multiple "message" displays and other cosmetic changes.

   Version 1.5:    October 14, 1998
                   Added OPTION to add multiuser tables and CONVERT length.
                   ...Still working on the Referential Integrity issue :(                      

   Version 1.6:    October 19, 1998
                   Changed the PACK routine to use an APPEND FROM instead of
                   a COPY TO since we found out the hard way that COPY TO
                   does not copy DEFAULT, MIN, MAX & REQUIRED properties.
                   Although it DOES seem to copy CUSTOM properties, go figure...

   Version 1.7:    October 24, 1998
                   Changed the APPEND FROM routine to use a custom rowset version
                   since it was also determined that APPEND FROM _IGNORES_ the
                   "DELETED" setting and brings deleted records back in anyway!
                   This also iliminated the need to re-CONVERT the database
                   after the pack since this method seems to copy the _LOCK field.

   Version 1.8:    Added code to extract the ALIAS PATH from the database.  
                   This should allow the program work properly
                   on tables that are not stored in the directory from which
                   the program is executing (i.e. BDE Aliases).

                   MANY Thanks to Bruno Bicanic for the above code suggestions

                   You now have the option of selecting the DATABASE you wish
                   to generate the program for when calling MAKEINDEXLIB.PRG.


   This program is designed to create a program file that will recreate all
   index tags if they _DO NOT EXIST_.  This is different than the CREATE REINDEX
   version created by Ken Mayer in the following ways:

   You do not have to execute this routine from within a working
   application.  It should be the _FIRST_ routine called in your application.
   Do this literally FIRST, before opening any forms, security checks, etc.

   This routine is different because you do not have to be able to successfully 
   OPEN the table for this routine to work.  Ken's CREATE REINDEX program
   requires that the index tag be still attached to the table.  In an 
   "worst-case" senerio, index corruption may be so bad, that you would
   not be able to gain access to an application to execute the REINDEX.PRG;
   such as in the case of a security system table which limits access
   to the application or parts thereof.  Also, if hard drive corruption deletes
   the MDX file (or a user does, what are the chances <g>), Ken's CREATE REINDEX
   program would not be able to help.

   This routine allows you (in a worst-case senerio), to DEL *.MDX from a
   command prompt, run your application, and have all tags rebuilt from scratch.
   This also means that MDX files do not need to be distributed with the
   application at deployment time.

   This routine will create a single program file that must be included
   in your application.  It should be the _FIRST_ routine called when your
   application is executed.

   If you add a table, or add a new index tag, or change an index tag 
   definition, you need to re-run this routine!

   USAGE:   SET PROCEDURE TO INDEXLIB.PRG (in your start.prg)
   
   CALLS:   CheckIndexes() - Checks/Creates missing index tags
                             Call FIRST thing in your app (usually)
   
            ReindexAll()   - Deletes all MDX's and reindexes
                             Call from a MENU option (usually)
   
            RePackAll()    - Packs all tables, removes memo bloat and reindexes
                             Call from a MENU option (usually)
   
   NOTE:    Suggestion are ALWAYS welcome!  Although I consider myself an
            expert programmer of "the old way", I am still learning VDB 7.01!

            The _created_ INDEXLIB.PRG also uses JMESSAGE.WFM for displaying
            status messages to the user.  It is a very basic message form
            that is included as a class at the end of INDEXLIB.PRG.
            If you like it, leave it there. If you don't like it, change it.

   THANKS:  To all people on the newsgroups who have given suggestions as to
            the proper usage of the many steps required to do this.  Especially 
            those suggestions related to the many undocumented features REQUIRED
            to make this work.

   WARNING: If you assign LookupSQL stataments (or the such) as custom 
            properties to any tables, the order in which the tables are
            reindexed is VERY IMPORTANT.

            FOR EXAMPLE:

            If you have a "custom databse property" of LookupSQL defined,
            you will need to be sure that the index is generated for the
            _LOOKUP_ before you attempt to rebuild the indexes for the
            PARENT.  Otherwise when the parent is opened, it will try to 
            open the _lookup(index)_ and fail with an MDX detached message.

   WARNING: This program will NOT WORK if you are using REFERENTAL INTEGRITY.
            (This support will be added soon)

   WANTED:  1) If someone could come up with an OODML way to rebuild the PRIMARY
               INDEX, it would be a great improvement.  I had to resort to the old
               DML way of doing PRIMARY KEY rebuildings.

            2) A way to save/restore REFERENTIAL INTEGRITY


   AUTHOR:  Jay C. Heil      Rider@mediaone.net

            Major Contributions From:
            Bruno Bicanic    (do you mind your e-mail here?)

            Other Contributions From:
            Gary White       (do you mind your e-mail here?)

            (and many other contributers as mentioned/forgotton above/below)


            This file is USUALLY at: ftp://ftp.fspalace.dynip.com/pub/vdbase
            (as well as many other VDB servers/sites)
            
*/

#DEFINE alltrim(x)   upper(ltrim(rtrim(x)))
#DEFINE WIN32API_REG_H                       // for registry
#INCLUDE <win32api.h>

extern CLONG RegCloseKey( HKEY ) ADVAPI32
extern CLONG RegOpenKeyEx( HKEY, LPCTSTR, DWORD, REGSAM, PHKEY ) ADVAPI32 from "RegOpenKeyExA" 
extern CLONG RegQueryValueEx( HKEY, LPTSTR, DWORD, LPDWORD, CSTRING, LPDWORD ) ADVAPI32 from "RegQueryValueExA"

LOCAL indentlevel
LOCAL i
LOCAL cPath 
LOCAL f

indentlevel = 0
close tables // just to be sure none are open ...

// ask for the DATABASE(alias) we wish to use...
cDatabase = Question("Enter a DATABASE(alias) NAME or leave BLANK for current directory")

// load table info into an array
// if a DATABASE was chosen, use it, otherwise use the current directory
if empty(cDatabase)
   cPath = ""
else
   cPath = FindPath( cDatabase )
   if cPath = "**ERROR**"
      return
   endif
endif

aT_Tables = new Array()
aT_Tables.dirExt( cPath + "*.dbf" )

// check to see if we're wasting our time:
if aLen( aT_Tables, 1 ) = 0
   msgbox( "There are no .DBF tables in this directory/database!",;
           "Nothing to do", 64)
   return
endif

// resize the array so it has only one column,
// that being the names of the tables:
nRows = aLen( aT_Tables,1 )
aT_Tables.resize( nRows, 1, 1)

// get rid of any _ "temp" files & KEYVIOLATION files
aTables = new ARRAY()
for i = 1 to alen( aT_Tables )
   if left(aT_Tables[i],1) <> "_" .and. left(aT_Tables[i],4) <> "KEYV" .and. aT_Tables[i] <> "TEMP.DBF"
      aTables.add( alltrim(aT_Tables[i]) )
   endif
next i

aT_Tables = null
TheDBaseName = ""

// allow the user to decide the order of the reindex
// SEE IMPORTANT NOTE ABOVE AS TO WHY!!!
if ReOrderArray( aTables, cDatabase ) = NULL
   msgbox('Program Aborted','QUIT',64)
   return
endif

// create file object:
fProgram = new File()
// should just overwrite one if it exists
fProgram.create( "indexlib.prg", "W" )
// Store some basic stuff into beginning of program:
cOutString = "/*"
fProgram.puts( space(indentlevel)+cOutString )
cOutString = "   INDEXLIB.PRG"
fProgram.puts( space(indentlevel)+cOutString )
cOutString = "   Program Generated by 'MakeIndexLib.prg'  Version 1.8"
fProgram.puts( space(indentlevel)+cOutString )
cOutString = "   "+date()+" - "+time()
fProgram.puts( space(indentlevel)+cOutString )
cOutString = "   "
fProgram.puts( space(indentlevel)+cOutString )
cOutString = "   AUTHOR:  Jay C. Heil      Rider@mediaone.net"
fProgram.puts( space(indentlevel)+cOutString )
cOutString = "            (See Makeindexlib.prg source for other credits)"
fProgram.puts( space(indentlevel)+cOutString )
cOutString = "   "
fProgram.puts( space(indentlevel)+cOutString )
cOutString = "   CheckIndexes() should be the FIRST executed procedure in"
fProgram.puts( space(indentlevel)+cOutString )
cOutString = "   your application.  This will recreate all index tags if"
fProgram.puts( space(indentlevel)+cOutString )
cOutString = "   they do not exist.  Therefore, DEL *.MDX before running"
fProgram.puts( space(indentlevel)+cOutString )
cOutString = "   your application; to fix index corruption."
fProgram.puts( space(indentlevel)+cOutString )
cOutString = "   "
fProgram.puts( space(indentlevel)+cOutString )
cOutString = "   This program is USUALLY at: ftp://ftp.fspalace.dynip.com/pub/vdbase"
fProgram.puts( space(indentlevel)+cOutString )
cOutString = "   (as well as many other VDB servers/sites)   "
fProgram.puts( space(indentlevel)+cOutString )
cOutString = "   "
fProgram.puts( space(indentlevel)+cOutString )
cOutString = "   USAGE:"
fProgram.puts( space(indentlevel)+cOutString )
cOutString = "         SET PROCEDURE TO INDEXLIB.PRG (in your start.prg)"
fProgram.puts( space(indentlevel)+cOutString )
cOutString = "   "
fProgram.puts( space(indentlevel)+cOutString )
cOutString = "   CALLS:   "
fProgram.puts( space(indentlevel)+cOutString )
cOutString = "         CheckIndexes() - Checks/Creates missing index tags"
fProgram.puts( space(indentlevel)+cOutString )
cOutString = "                          Call FIRST thing in your app (usually)"
fProgram.puts( space(indentlevel)+cOutString )
cOutString = "   "
fProgram.puts( space(indentlevel)+cOutString )
cOutString = "         ReindexAll()   - Deletes all MDX's and reindexes"
fProgram.puts( space(indentlevel)+cOutString )
cOutString = "                          Call from a MENU option (usually)"
fProgram.puts( space(indentlevel)+cOutString )
cOutString = "   "
fProgram.puts( space(indentlevel)+cOutString )
cOutString = "         RePackAll()    - Packs all tables, removes memo bloat and reindexes"
fProgram.puts( space(indentlevel)+cOutString )
cOutString = "                          Call from a MENU option (usually)"
fProgram.puts( space(indentlevel)+cOutString )
cOutString = "   "
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [   WARNING: If you assign LookupSQL stataments as custom properties to the DB,]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [            the order in which the tables are reindexed is VERY IMPORTANT.]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [            FOR EXAMPLE:]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [            If you have a "custom databse property" of LookupSQL defined,]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [            you will need to be sure that the index is generated for the]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [            _LOOKUP_ before you attempt to rebuild the indexes for the]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [            PARENT.  Otherwise when the parent is opened, it will try to ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [            open the _lookup_ and fail with a MDX detached message.]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [            **************************************************************]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [            **  IF, WHEN RUNNING ANY OF THESE FUNCTIONS, YOU RECEIVE    **]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [            **  AN ERROR RELATING TO "FAILED TO OPEN TABLE/INDEX/ETC",  **]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [            **  THEN THE "ORDER" IN WHICH YOUR TABLES ARE INDEXED IS    **]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [            **  RELEVANT AND MUST BE CHANGED THE NEXT TIME THAT YOU     **]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [            **  RUN MAKEINDEXLIB.PRG.  ALL TABLES THAT ARE "LOOKUPS"    **]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [            **  _to_ ANOTHER TABLE, _must_ COME _first_.                **]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [            **************************************************************]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = "*/"
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = '#define WIN32API_REG_H    // for registry'
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [#include <win32api.h>]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = "FUNCTION CheckIndexes"
fProgram.puts( space(indentlevel)+cOutString )
cOutString = ""
fProgram.puts( space(indentlevel)+cOutString )
cOutString = "LOCAL filesOK"
fProgram.puts( space(indentlevel)+cOutString )
cOutString = "LOCAL cPath"
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [extern CLONG RegCloseKey( HKEY ) ADVAPI32]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [extern CLONG RegOpenKeyEx( HKEY, LPCTSTR, DWORD, REGSAM, PHKEY ) ADVAPI32 from "RegOpenKeyExA" ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [extern CLONG RegQueryValueEx( HKEY, LPTSTR, DWORD, LPDWORD, CSTRING, LPDWORD ) ADVAPI32 from "RegQueryValueExA"]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = "fMessage = new messageForm()"
fProgram.puts( space(indentlevel)+cOutString )
cOutString = "fMessage.text = 'Table Verification'"
fProgram.puts( space(indentlevel)+cOutString )
cOutString = "fMessage.messageText.text = 'Checking for Tables'"
fProgram.puts( space(indentlevel)+cOutString )
cOutString = "fMessage.open()"
fProgram.puts( space(indentlevel)+cOutString )
cOutString = ""
fProgram.puts( space(indentlevel)+cOutString )
cOutString = "FilesOK = true"
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [cPath   = FindPath("]+TheDbaseName+[")]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = ""
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [if cPath = "**ERROR**"]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [   fMessage.release()]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [   RETURN false]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [elseif cPath = "\"]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [  cpath = ""]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [endif]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = ""
fProgram.puts( space(indentlevel)+cOutString )
cOutString = "CLOSE TABLES"
fProgram.puts( space(indentlevel)+cOutString )
cOutString = ""
fProgram.puts( space(indentlevel)+cOutString )
cOutString = "// check that all the DB's exist for this application"
fProgram.puts( space(indentlevel)+cOutString )


for tableNumber = 1 to aLen( aTables, 1 )
   cOutString = "filesOK := DBok( cPath+'"+alltrim(aTables[tableNumber])+"' ) .AND. filesOK"
   fProgram.puts( space(indentlevel)+cOutString )
next tableNumber


cOutString = ""
fProgram.puts( space(indentlevel)+cOutString )
cOutString = "// Create any needed index tags (only for tables _with_) an index"
fProgram.puts( space(indentlevel)+cOutString )

cOutString = "IF filesOK"
fProgram.puts( space(indentlevel)+cOutString )

indentlevel += 3

cOutString = "fMessage.text = 'Index Verification'"
fProgram.puts( space(indentlevel)+cOutString )
cOutString = ""
fProgram.puts( space(indentlevel)+cOutString )


for tableNumber = 1 to aLen( aTables, 1 )
   // Instance of TableDef object
   tTable = new TableDef() // undocumented feature
   // Give it the table name to use
   tTable.tableName := cPath + aTables[tableNumber]
   // Load the table
   tTable.load()

   // Loop through index tags, if any. 
   if tTable.indexes.size > 0

      for indexNumber = 1 to tTable.indexes.size
         cOutString = "IndexOK( cPath+["+alltrim(aTables[tableNumber])+ ;
                      "] , ["+alltrim(tTable.indexes[indexNumber].indexName)+ ;
                      "] , ["+alltrim(tTable.indexes[indexNumber].expression)+ ;
                      "] , ["+alltrim(tTable.indexes[indexNumber].forExpression)+ ;
                      "] , "+tTable.indexes[indexNumber].descending+ ;
                       " , "+tTable.indexes[indexNumber].unique+ ;
                       " , ["+alltrim(tTable.primaryKey)+"] )"
         fProgram.puts( space(indentlevel)+cOutString )
      next indexNumber

   else

      cOutString = "// No indexes defined For Table: "+aTables[tablenumber]
      fProgram.puts( space(indentlevel)+cOutString )

   endif

next tableNumber


indentlevel -= 3

cOutString = "ENDIF"
fProgram.puts( space(indentlevel)+cOutString )
cOutString = ""
fProgram.puts( space(indentlevel)+cOutString )
cOutString = "fMessage.release()"
fProgram.puts( space(indentlevel)+cOutString )
cOutString = ""
fProgram.puts( space(indentlevel)+cOutString )
cOutString = "RETURN filesOK"
fProgram.puts( space(indentlevel)+cOutString )
cOutString = ""
fProgram.puts( space(indentlevel)+cOutString )
cOutString = "*---------------------------------------------------------------------"
fProgram.puts( space(indentlevel)+cOutString )
cOutString = ""
fProgram.puts( space(indentlevel)+cOutString )




*------------------------------------------------------------------------------

cOutString = [FUNCTION IndexOK ( dbf_, tagname_, exp_, condition_, Dec_Index_, isUnique_, PrimaryKey_ )]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [LOCAL reindex_]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [LOCAL t]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [LOCAL matchfound]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = "fMessage.messageText.text = 'Checking Indexes For Table: '+upper(dbf_)"
fProgram.puts( space(indentlevel)+cOutString )
cOutString = "fMessage.Repaint()"
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [tagname_ = UPPER(tagname_)]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [needReindex = false]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [matchfound = false]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [mdxname = left(dbf_,len(dbf_)-4)+".MDX"]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = '   // check if the MDX file _itself_ exists... '
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [   if .not. file( mdxname )]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = '      // MDX does not exist, it must have been deleted.'
fProgram.puts( space(indentlevel)+cOutString )
cOutString = '      // Flip the MDX BYTE in the DBF so we can open it without'
fProgram.puts( space(indentlevel)+cOutString )
cOutString = '      // having it bitch that the MDX is gone.'
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [      if FlipMDXByte( dbf_ )]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [         needReindex = true]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [      else]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [         RETURN false]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [      endif]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [   endif]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [   if .not. needReindex]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = '      // setup a tableDef so we can check if the tag already exists...'
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [      t = new TableDef()]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [      t.tableName   := dbf_]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [      t.load()]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = '      // see if we can find the tag...'
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [    ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [      for i = 1 to t.indexes.size]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = '        if tagname_ = upper(t.indexes[i].indexName) '
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [            i := t.indexes.size + 1 ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [            matchfound = true]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [        endif]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [      next]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [   endif]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = '   // if the tag is gone (or newly requested), create it...'
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [   if (.not. matchfound) .or. (needReindex)]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [      fMessage.text := 'Index Verification']
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [      fMessage.messageText.text := 'Rebuilding Index: ' + tagname_ + ' for Table: ' + dbf_ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [      fMessage.Repaint()]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [  ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [      if PrimaryKey_ == Exp_]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [         indxstr = alltrim(exp_)+' TAG "'+alltrim(tagname_)+'" ']
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [         if .not. empty(condition_)]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [            indxstr += 'FOR '+alltrim(condition)+' ']
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [         endif]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [         if Dec_Index_]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [            indxstr += "DESCENDING "]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [         endif]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [         if isUnique_]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [            indxstr += "UNIQUE "]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [         endif]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [         indxstr += "PRIMARY "]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [         use (dbf_) EXCLUSIVE]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [         INDEX ON &indxstr]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [         use]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [      else]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [         d = new DBFIndex() ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [         d.expression    := exp_]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [         d.descending    := Dec_Index_]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [         d.unique        := isUnique_]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [         d.forExpression := condition_]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [         d.indexName     := tagname_]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = '         _app.databases[1].createIndex( dbf_, d ) '
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [      endif]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [   endif]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [RETURN true]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [*------------------------------------------------------------------------------]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [FUNCTION DBok ( dbname_ )]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [fMessage.text := 'Table Verification']
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [fMessage.messageText.text := 'Checking Table: '+dbname_]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [fMessage.Repaint()]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [if .not. file( dbname_ )]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [   msgbox('Table: ' + dbname_ + ' is missing','Table Verification',48)]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [   return false]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [endif]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [RETURN true]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [*------------------------------------------------------------------------------]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [FUNCTION FlipMDXByte( dbf_ )]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [LOCAL f]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [LOCAL success]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [success = true]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [f = new FILE()]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [TRY]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [   f.open( dbf_, "RW" )]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [   if f.seek( 28, 0 ) <> 28]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [     success := false]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [     msgbox('Cannot locate MDX byte in the DBF file: ' + dbf_ ,'Error searching for MDX byte',48)]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [   else]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [      if (f.write( chr(0) ) <> 1) .or. (f.error() <> 0)]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [        success := false]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [        msgbox('Cannot write to the DBF file: ' +dbf_  ,'Error setting MDX byte',48)]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [      endif]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [   endif]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [CATCH (exception e)]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [   success := false]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [   msgbox('Cannot open the DBF file: ' +dbf_ ,'Error opening DBF file',48)]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ENDTRY]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [f.close()]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [RETURN success]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [*------------------------------------------------------------------------------]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [FUNCTION ReindexAll]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [LOCAL filesOk ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [LOCAL cPath ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [extern CLONG RegCloseKey( HKEY ) ADVAPI32]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [extern CLONG RegOpenKeyEx( HKEY, LPCTSTR, DWORD, REGSAM, PHKEY ) ADVAPI32 from "RegOpenKeyExA" ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [extern CLONG RegQueryValueEx( HKEY, LPTSTR, DWORD, LPDWORD, CSTRING, LPDWORD ) ADVAPI32 from "RegQueryValueExA"]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [filesOk = .t. ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [cPath   = FindPath("]+TheDbaseName+[")]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [if cPath = "**ERROR**"]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [   RETURN false]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [elseif cPath = "\"]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [  cpath = ""]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [endif]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = ""
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [if msgbox('Reindexing requires that all users be out of this application!'+chr(13)+'Are you ready to continue?','Reindex All Tables',36) = 6]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [   if msgbox('Verify that YOUR machine does not have ANY open windows in this application!'+chr(13)+'Are you ready to continue?','Reindex All Tables',36) = 6]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [      CLOSE TABLES]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [      SET SAFETY OFF]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )

indentlevel = 6
cOutString = "fMessage = new messageForm()"
fProgram.puts( space(indentlevel)+cOutString )
cOutString = "fMessage.text = 'Reindex all Tables'"
fProgram.puts( space(indentlevel)+cOutString )
cOutString = "fMessage.messageText.text = 'Deleting Indexes'"
fProgram.puts( space(indentlevel)+cOutString )
cOutString = "fMessage.open()"
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = '// erase all of the MDX files for this application'
fProgram.puts( space(indentlevel)+cOutString )


for tableNumber = 1 to aLen( aTables, 1 )
   temp = upper(aTables[tablenumber])
   temp = left(temp,len(temp)-4)+".MDX"

   cOutString = [filesOK := DelTag( cPath+"] + temp + [" ) .and. FilesOk ]
   fProgram.puts( space(indentlevel)+cOutString )

next tableNumber

indentLevel := 6
cOutString = [fMessage.release()]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [if .not. filesOK ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [   msgbox('Could not open files exclusively','Notice',48)]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [   CLOSE ALL]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [   QUIT]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [endif ]
fProgram.puts( space(indentlevel)+cOutString )

indentLevel := 0
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [      if .not. CheckIndexes()]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [         msgbox('Files were missing, cannot continue','Notice',48)]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [         CLOSE ALL]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [         QUIT]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [      endif]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [      msgbox('Reindex Complete','Notice',48)]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [       ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [   endif]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [endif]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [RETURN]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [*------------------------------------------------------------------------------]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [FUNCTION RePackAll]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [LOCAL t]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [LOCAL packOk]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [LOCAL filesOk ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [LOCAL thePath ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [LOCAL cPath ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [extern CLONG RegCloseKey( HKEY ) ADVAPI32]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [extern CLONG RegOpenKeyEx( HKEY, LPCTSTR, DWORD, REGSAM, PHKEY ) ADVAPI32 from "RegOpenKeyExA" ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [extern CLONG RegQueryValueEx( HKEY, LPTSTR, DWORD, LPDWORD, CSTRING, LPDWORD ) ADVAPI32 from "RegQueryValueExA"]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [packOk = true]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [filesOk = .t. ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [cPath   = FindPath("]+TheDbaseName+[")]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [if cPath = "**ERROR**"]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [   RETURN false]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [elseif cPath = "\"]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [  cpath = ""]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [endif]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = []
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [if msgbox('WARNING: Be sure that you have a current BACKUP before continuing!'+chr(13)+'Are you ready to continue?','Pack All Tables',36) = 6]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [   if msgbox('Packing requires that all users be out of this application!'+chr(13)+'Are you ready to continue?','Pack All Tables',36) = 6]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [      if msgbox('Verify that YOUR machine does not have ANY open windows in this application!'+chr(13)+'Are you ready to continue?','Pack All Tables',36) = 6]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [         CLOSE TABLES]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [         SET SAFETY OFF]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [     ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [         fMessage = new messageForm()]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [         fMessage.text = 'Deleting all indexes']
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [         fMessage.messageText.text = 'Deleting Indexes']
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [         fMessage.open()]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [           ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = '         // erase all of the MDX files for this application'
fProgram.puts( space(indentlevel)+cOutString )

indentlevel := 9
for tableNumber = 1 to aLen( aTables, 1 )
   temp = upper(aTables[tablenumber])
   temp = left(temp,len(temp)-4)+".MDX"

   cOutString = [filesOK := DelTag( cPath+"] + temp + [" ) .and. FilesOk ]
   fProgram.puts( space(indentlevel)+cOutString )

next tableNumber

cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [if .not. filesOK ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [   msgbox('Could not open files exclusively','Notice',48)]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [   fMessage.release()]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [   CLOSE ALL]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [   QUIT]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [endif ]
fProgram.puts( space(indentlevel)+cOutString )

indentLevel := 0

cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = '         // Now perform the actual packing using the CUSTOM APPEND method.'
fProgram.puts( space(indentlevel)+cOutString )
cOutString = '         // This gets rid of memo field bloat and also fixes a number of'
fProgram.puts( space(indentlevel)+cOutString )
cOutString = '         // database oriented errors and corruption.'
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [         CLOSE TABLES]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [         fMessage.text = 'Packing all Tables']
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )


indentlevel = 9 
for tableNumber = 1 to aLen( aTables, 1 )

   cOutString = "packOk = PackTable( cPath, '" + alltrim(aTables[tablenumber]) + "', '"+alltrim(TheDBaseName)+"' ) .and. packOk"
   fProgram.puts( space(indentlevel)+cOutString )
   
next tableNumber

indentLevel = 0
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [         fMessage.release()]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [     ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = '         // packing is all done, now reindex the tables...'
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [         if PackOk]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [            if .not. CheckIndexes()]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [               msgbox('Files were missing, cannot continue','Notice',48)]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [               CLOSE ALL]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [               QUIT]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [            endif]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [         else]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [            msgbox('Packing could not be completed, cannot continue','Notice',48)]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [            CLOSE ALL]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [            QUIT]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [         endif]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [         msgbox('Packing/Reindex Complete','Notice',48)]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [          ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [      endif]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [   endif]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [endif]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [RETURN]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [*------------------------------------------------------------------------------]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [FUNCTION IndexLib ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [   MSGBOX('You do not need to directly execute this program file.  See SOURCE for help')]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [RETURN ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [*------------------------------------------------------------------------------]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [Function DelTag ( TheTag ) ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = "fMessage.messageText.text = 'Deleting Index: '+ TheTag"
fProgram.puts( space(indentlevel)+cOutString )
cOutString = "fMessage.Repaint()"
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )

cOutString = "try"
fProgram.puts( space(indentlevel)+cOutString )
indentlevel += 3

cOutString = "ERASE ( TheTag )"
fProgram.puts( space(indentlevel)+cOutString )

indentlevel -= 3
cOutString = "catch (exception e)"
fProgram.puts( space(indentlevel)+cOutString )

indentlevel += 3
cOutString = [RETURN false ]
fProgram.puts( space(indentlevel)+cOutString )

indentlevel -= 3
cOutString = "endtry"
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )

cOutString = [RETURN true ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )

cOutString = [*------------------------------------------------------------------------------]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [function PackTable( cPath, cDBFname, cDbaseName )]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [local d, q, u, cDBTname]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [fMessage.messageText.text = 'Packing File: '+ cPath + cDBFname]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [fMessage.Repaint()]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [cDBTname = left(cDBFname, len(cDBFname)-3) + "DBT"]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [if file( cPath + cDBTname )]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [   new File().copy( cPath + cDBTName, cPath + "TEMP.DBT" )]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [endif ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [new File().copy( cPath + cDbfName, cPath + "TEMP.DBF" )]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [if FlipMDXByte( cPath + "TEMP.DBF" ) .and. FlipMDXByte( cPath + cDBFname )]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [if empty( cDbaseName ) ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = '   _app.databases[ 1 ].emptyTable( cDBFname )'
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [else]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [   d = new database()]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = "   d.databaseName = cDbaseName      ///// here was mistake"
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [   d.active = true]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [   d.emptyTable( cDBFname )]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [endif]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [q = new Query()]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [if not empty( cDbaseName ) ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [   q.database = d]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [endif]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [q.sql = "select * from TEMP"]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [q.active = true]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [u = new UpdateSet()]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [u.source = q.rowset]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [u.destination = cPath + cDBFname]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [u.append()]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [q.active = false]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [if empty( cDbaseName ) ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = '   _app.databases[ 1 ].dropTable( "Temp" ) '
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [else]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [   d.dropTable( "Temp" )]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [   d.active = false]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [   d = null]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [endif]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [q = null]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [u = null]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [RETURN true]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [endif]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [return false]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [*------------------------------------------------------------------------------]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = 'class messageForm of FORM'
fProgram.puts( space(indentlevel)+cOutString )
cOutString = '   with (this)'
fProgram.puts( space(indentlevel)+cOutString )
cOutString = '      canClose = {|| false }'
fProgram.puts( space(indentlevel)+cOutString )
cOutString = '      scaleFontBold = false'
fProgram.puts( space(indentlevel)+cOutString )
cOutString = '      height = 4.2727'
fProgram.puts( space(indentlevel)+cOutString )
cOutString = '      left = 99.8571'
fProgram.puts( space(indentlevel)+cOutString )
cOutString = '      top = 15.1818'
fProgram.puts( space(indentlevel)+cOutString )
cOutString = '      width = 44.8571'
fProgram.puts( space(indentlevel)+cOutString )
cOutString = '      text = "Processing..."'
fProgram.puts( space(indentlevel)+cOutString )
cOutString = '      topMost = true'
fProgram.puts( space(indentlevel)+cOutString )
cOutString = '      autoCenter = true'
fProgram.puts( space(indentlevel)+cOutString )
cOutString = '      mdi = false'
fProgram.puts( space(indentlevel)+cOutString )
cOutString = '      smallTitle = true'
fProgram.puts( space(indentlevel)+cOutString )
cOutString = '      sizeable = false'
fProgram.puts( space(indentlevel)+cOutString )
cOutString = '      moveable = false'
fProgram.puts( space(indentlevel)+cOutString )
cOutString = '      escExit = false'
fProgram.puts( space(indentlevel)+cOutString )
cOutString = '      maximize = false'
fProgram.puts( space(indentlevel)+cOutString )
cOutString = '      minimize = false'
fProgram.puts( space(indentlevel)+cOutString )
cOutString = '   endwith'
fProgram.puts( space(indentlevel)+cOutString )
cOutString = ' '
fProgram.puts( space(indentlevel)+cOutString )
cOutString = ' '
fProgram.puts( space(indentlevel)+cOutString )
cOutString = '   this.MESSAGETEXT = new TEXT(this)'
fProgram.puts( space(indentlevel)+cOutString )
cOutString = '   with (this.MESSAGETEXT)'
fProgram.puts( space(indentlevel)+cOutString )
cOutString = '      height = 3.9091'
fProgram.puts( space(indentlevel)+cOutString )
cOutString = '      left = 0'
fProgram.puts( space(indentlevel)+cOutString )
cOutString = '      top = 0.1818'
fProgram.puts( space(indentlevel)+cOutString )
cOutString = '      width = 44.8571'
fProgram.puts( space(indentlevel)+cOutString )
cOutString = '      alignVertical = 1	// Middle'
fProgram.puts( space(indentlevel)+cOutString )
cOutString = '      alignHorizontal = 1	// Center'
fProgram.puts( space(indentlevel)+cOutString )
cOutString = '      text = "Display Message"'
fProgram.puts( space(indentlevel)+cOutString )
cOutString = '   endwith'
fProgram.puts( space(indentlevel)+cOutString )
cOutString = ' '
fProgram.puts( space(indentlevel)+cOutString )
cOutString = ' '
fProgram.puts( space(indentlevel)+cOutString )
cOutString = '   function Repaint'
fProgram.puts( space(indentlevel)+cOutString )
cOutString = '      if type( "UpdateWindow" ) # "FP"'
fProgram.puts( space(indentlevel)+cOutString )
cOutString = '         extern CLOGICAL UpdateWindow( CHANDLE ) USER32'
fProgram.puts( space(indentlevel)+cOutString )
cOutString = '      endif'
fProgram.puts( space(indentlevel)+cOutString )
cOutString = '      // if no parameter, use current form'
fProgram.puts( space(indentlevel)+cOutString )
cOutString = '      if argCount() = 0'
fProgram.puts( space(indentlevel)+cOutString )
cOutString = '         UpdateWindow( form.hWnd )'
fProgram.puts( space(indentlevel)+cOutString )
cOutString = '      else'
fProgram.puts( space(indentlevel)+cOutString )
cOutString = '         UpdateWindow( argVector(1) )'
fProgram.puts( space(indentlevel)+cOutString )
cOutString = '      endif'
fProgram.puts( space(indentlevel)+cOutString )
cOutString = ' '
fProgram.puts( space(indentlevel)+cOutString )
cOutString = '   RETURN '
fProgram.puts( space(indentlevel)+cOutString )
cOutString = 'endclass'
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [*------------------------------------------------------------------------------]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [function FindPath( cDbaseName )]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [LOCAL nDbaseLen]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [LOCAL x]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [LOCAL cSekvence]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [LOCAL BDEpath]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [nDbaseLen  = len( cDbaseName )]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [if len(cDbaseName) = 0]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = '  RETURN ""      // not using an ALIAS, return NOTHING...'
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [endif]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [BDEpath = GetBDEPath()]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [if BDEpath = "**ERROR**"]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [   RETURN "**ERROR**"]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [endif]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [f = new FILE()]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [TRY]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [f.open( BDEpath, "R" )]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [   x = 15]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [   cSekvence = ""]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = "   // Find Database name in config file"
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [   do while ( cSekvence # cDbaseName .and. .not. f.eof() )]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [      x++]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [      f.seek( x, 0 )]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [      cSekvence := f.gets( nDbaseLen, chr( 0x00 ) )]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [   enddo]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [   if empty(cSekvence)]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [     msgbox('Cannot locate database '+cDbaseName+' in BDE configuration file','Looking for BDE Alias',48)]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [     f.close()]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [     RETURN "**ERROR**"]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [   else]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = "     // add len of database name + 27 empty places to start before PATH"
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [     x += ( nDbaseLen + 27 )]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = '     // Find "PATH" in config file'
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [     do while ( cSekvence # "PATH" .and. .not. f.eof() )]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [        x++]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [        f.seek( x, 0 )]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [        cSekvence := f.gets( 4, chr( 0x00 ) )]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [     enddo]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = '     // add len of "PATH" (4) + 3 empty places'
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [     f.seek( x+7, 0 )]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = "     // here is a path for database"
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [     cSekvence = f.gets( 100, chr( 0x00 ) )]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [   endif]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [CATCH (exception e) ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [   msgbox('Cannot open BDE configuration file','Looking for BDE Alias',48)]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [   f.close()      ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [   RETURN "**ERROR**"]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ENDTRY ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [f.close()      ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [RETURN cSekvence + "\"]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [*------------------------------------------------------------------------------]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [FUNCTION GetBDEpath()]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [LOCAL openKey]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [LOCAL openSubKey]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [LOCAL nKey]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [LOCAL nOK]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [LOCAL nLen]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [LOCAL nType]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [LOCAL cData]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [openKey    = HKEY_LOCAL_MACHINE]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [openSubKey = "SOFTWARE\Borland\Database Engine"]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = "// OPEN KEY"
fProgram.puts( space(indentlevel)+cOutString )
cOutString = "nKey = 0            // handle of new key "
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [nOK = RegOpenKeyEx( openKey, openSubKey, 0, KEY_ALL_ACCESS, nKey )]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [if not (nOK == ERROR_SUCCESS) ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [   msgbox( "Cannot open BDE Primary KEY in registry",;]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [           "Error Finding BDE Path", 64)]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [   RegCloseKey( nKey )]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [   return "**ERROR**"]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [endif]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = "// READ SUBKEY VALUE"
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [nOK   = 0 ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [nType = 0]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [nLen  = 80]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [cData = REPLICATE(" ", 80) ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [nOK   = RegQueryValueEx( nKey, "CONFIGFILE01", 0, nType, cData, nLen ) ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [if not (nOK == ERROR_SUCCESS)]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [   msgbox( "Cannot open BDE Path KEY in registry",;]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [           "Error Finding BDE Path", 64)]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [   RegCloseKey( nKey )]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [   return "**ERROR**"]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [endif]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = "// CLOSE KEY"
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [RegCloseKey( nKey )]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [RETURN (cData)]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [*------------------------------------------------------------------------------]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = [ ]
fProgram.puts( space(indentlevel)+cOutString )
cOutString = '// End of INDEXLIB.PRG '
fProgram.puts( space(indentlevel)+cOutString )

msgbox('Complete. INDEXLIB.PRG generated')

fprogram.close()

RETURN

*------------------------------------------------------------------------------

FUNCTION ReOrderArray ( aWork, cDatabase )

   // allow the user to decide the order of the tables for the reindex...

   aWork.sort()   // start out as a sorted array

   LOCAL f

   f = new listBoxDragForm()
   f.aSource = aWork
   f.TheDBaseName.value = cDatabase
   f.mdi = false // ensure not MDI
   f.readModal()

   aWork := f.aSource
   TheDBaseName = f.TheDBaseName.value

   f.release()
   f = null

RETURN aWork

*------------------------------------------------------------------------------

/*

   Class: LISTBOXDRAWFORM

   Modified: By Jay C. Heil (rider@mediaone.net)
             for specific use in MAKEINDEXLIB.PRG
  
   Original Version Date:  April 1998, LISTBOX.WFM
   Original Written By:  David Peters  (dpeters@frisbee.net.au)
  
   For Version: Visual dBASE 7.01
                NOTE: This form does not work correctly in version 7.00
  
   Legal Stuff: Freeware.  Use it.  Hack it.  It's not my fault, etc.
                Please place credit where credit is due.
  
*/

class listboxDragForm of FORM
   with (this)
      onLeftMouseUp = class::FORM_ONLEFTMOUSEUP
      onMouseMove = class::FORM_ONMOUSEMOVE
      onOpen = class::FORM_ONOPEN
      scaleFontBold = false
      metric = 6	// Pixels
      height = 436
      left = 741
      top = 84
      width = 280
      text = "Table Ordering"
      autoCenter = true
      smallTitle = true
      sizeable = false
      mousePointer = 1	// Arrow
   endwith


   this.RECTANGLE1 = new RECTANGLE(this)
   with (this.RECTANGLE1)
      left = 7
      top = 316
      width = 264
      height = 78
      text = "BDE Alias Definition"
      fontBold = true
   endwith


   this.LISTBOX1 = new LISTBOX(this)
   with (this.LISTBOX1)
      onLeftMouseDown = class::LISTBOX1_ONLEFTMOUSEDOWN
      onLeftMouseUp = class::LISTBOX1_ONLEFTMOUSEUP
      onMouseMove = class::LISTBOX1_ONMOUSEMOVE
      height = 222
      left = 35
      top = 30
      width = 217
      id = 101
      mousePointer = 1	// Arrow
   endwith


   this.THEDBASENAME = new ENTRYFIELD(this)
   with (this.THEDBASENAME)
      height = 22
      left = 117
      top = 340
      width = 146
      function = "!"
      value = ""
   endwith


   this.CONTINUEBUTTON = new PUSHBUTTON(this)
   with (this.CONTINUEBUTTON)
      onClick = {; form.close()}
      height = 24
      left = 20
      top = 406
      width = 107
      text = "&Continue"
      value = false
      default = true
   endwith


   this.EXITBUTTON = new PUSHBUTTON(this)
   with (this.EXITBUTTON)
      onClick = class::EXITBUTTON_ONCLICK
      height = 24
      left = 151
      top = 406
      width = 107
      text = "E&xit"
      value = false
   endwith


   this.LISTBOXTOP = new ENTRYFIELD(this)
   with (this.LISTBOXTOP)
      when = {||false}
      onLeftMouseUp = class::FORM_ONLEFTMOUSEUP
      onMouseMove = class::LISTBOXTOP_ONMOUSEMOVE
      height = 22
      left = 35
      top = 12
      width = 217
      colorNormal = "btnface"
      border = false
      value = ""
      mousePointer = 1	// Arrow
      borderStyle = 3	// None
   endwith


   this.LISTBOXBOTTOM = new ENTRYFIELD(this)
   with (this.LISTBOXBOTTOM)
      when = {||false}
      onLeftMouseUp = class::FORM_ONLEFTMOUSEUP
      onMouseMove = class::LISTBOXBOTTOM_ONMOUSEMOVE
      height = 22
      left = 35
      top = 252
      width = 217
      colorNormal = "btnface"
      border = false
      value = ""
      mousePointer = 1	// Arrow
      borderStyle = 3	// None
   endwith


   this.TEXT1 = new TEXT(this)
   with (this.TEXT1)
      height = 48
      left = 12
      top = 258
      width = 256
      alignHorizontal = 1	// Center
      text = "Select the order in which you wish to have the tables reindexed.  See SOURCE CODE as to why this may be important."
   endwith


   this.TEXT2 = new TEXT(this)
   with (this.TEXT2)
      height = 24
      left = 4
      top = 6
      width = 272
      alignHorizontal = 1	// Center
      fontBold = true
      text = "Select the table order for reindexing"
   endwith


   this.TEXT3 = new TEXT(this)
   with (this.TEXT3)
      height = 22
      left = 15
      top = 342
      width = 100
      text = "Database Name:"
   endwith


   this.TEXT4 = new TEXT(this)
   with (this.TEXT4)
      height = 18
      left = 16
      top = 368
      width = 246
      alignVertical = 1	// Middle
      alignHorizontal = 1	// Center
      fontItalic = true
      text = "Leave BLANK if not using ALIASES"
   endwith


   function EXITBUTTON_onClick
      form.aSource := null
      form.close()
      return

   function form_onLeftMouseUp(flags, col, row)
      // If dragging then better tell the listbox that the left button is
      // not down anymore
      if form.leftDrag
         form.listbox1_onLeftMouseUp(-1)
      endif
   return

   function form_onMouseMove(flags, col, row)
      // Are we dragging?
      if form.leftDrag
         // Has left button been released?
         if not bitset(flags, 0)
            class::listbox1_onLeftMouseUp(-1)
         elseif form.timer1.enabled
            // We are still dragging but are on the form, better turn off
            // the timer to stop scrolling.
            form.timer1.enabled := false
         endif
      endif
   return

   function form_onOpen
      if type("mouse_event") # "FP"
         extern CVOID mouse_event( CWORD, CWORD, CWORD, CWORD, CWORD ) User32
      endif

      // Setup custom properties of the form
      form.leftDrag = false
      form.leftDragMove = false
      form.leftRow = 0

      // Create array to hold the listbox items
      form.items = form.aSource
      
      // Create a timer to do the off-screen listbox scrolling
      form.timer1 = new TIMER()
      form.timer1.parent    = form
      form.timer1.onTimer  := class::TIMER1_ONTIMER
      form.timer1.interval := 0.15
      form.timer1.direction = "DOWN"

      // Assign our array to the listbox and lets get cooking
      form.listBox1.dataSource = 'ARRAY form.items'
   return

   function LISTBOX1_onLeftMouseDown(flags, col, row)
      // Take note of the item being dragged
      form.leftSel = this.curSel

      // Put the form & listbox into dragging mode
      with (form)
         leftDrag = true
         listbox1.colorhighlight="white/gray"
         listbox1.mousePointer      := 7
         listboxTop.mousePointer    := 7
         listboxBottom.mousePointer := 7
         mousePointer               := 7
      endwith
   return                

   function LISTBOX1_onLeftMouseUp(flags, col, row)
      // Note dont refer to 'this' as function gets called by form when
      // 'this' is is out of scope.  Use form.listbox1 instead.

      // Turn off dragging
      form.leftDrag = false
      form.timer1.enabled := false
      form.listbox1.colorhighlight="white/navy"

      // If listbox item has been dragged to a new location then resort
      // the array and repaint it
      if form.leftSel # form.listBox1.curSel
         temp = form.items[form.leftSel]
         form.items.delete(form.leftSel)
         form.items.insert(form.listbox1.curSel)
         form.items[form.listbox1.curSel] = temp
         form.listbox1.dataSource += ""
      endif

      // Put the mouse pointers back to normal
      with (form)
         listBox1.mousePointer      := 1
         listboxTop.mousePointer    := 1
         listboxBottom.mousePointer := 1
         mousePointer               := 1
      endwith
   return

   function LISTBOX1_onMouseMove(flags, col, row)
      // Are we dragging
      if form.leftDrag
         // If left button is not depressed then stop dragging
         if not bitset(flags, 0)
            class::listbox1_onLeftMouseUp(-1)
         else
            // Turn off the timer for scrolling if we are in the listbox
            // as it is not needed here.
            if form.timer1.enabled
               form.timer1.enabled := false
            endif

            // The mouse has moved up or down about a row then press the
            // right mouse button on our current location to select a new
            // item in the listbox.

            // The 0.78 is a bit of a magic figure, it is about the height
            // of one listbox row in Arial 10 font.  Why it isn't 1 I dont
            // know!  If you are using different fonts/sizes you may need
            // to fine tune this figure.  Its main purpose it to only press
            // the right button when the mouse has moved about a row rather
            // than doing it all the time for every pixel of movement.
            if INT(row/0.78)+1 # this.curSel
               mouse_event(8,0,0,0,0) // right button down
               mouse_event(16,0,0,0,0) // right button up
            endif
         endif
      endif
   return

   function LISTBOXTOP_onMouseMove(flags, col, row)
      if form.leftDrag and bitSet(flags, 0)
         // If we are above the listbox and dragging then start scrolling
         // up if we can
         if not form.timer1.enabled and form.listbox1.curSel > 1
            form.timer1.direction := "UP"
            form.timer1.enabled   := true
         endif
      endif
   return

   function LISTBOXBOTTOM_onMouseMove(flags, col, row)
      if form.leftDrag and bitSet(flags, 0)
         // If we are below the listbox and dragging then start scrolling
         // down if we can
         if ( not form.timer1.enabled and ;
              form.listbox1.curSel < form.items.size )
            form.timer1.direction := "DOWN"
            form.timer1.enabled   := true
         endif
      endif
   return

   function TIMER1_onTimer
      // Lets be really pesimistic, if anything does not look right then
      // turn the timer off
      TRY
         do case
         case ( this.parent.activeControl.name == "LISTBOX1" ;
                and this.direction == "UP" )
            // Move the listbox up if we can
            if this.parent.listbox1.curSel > 1
               this.parent.listbox1.curSel--
            else
               this.enabled := false
            endif

         case ( this.parent.activeControl.name == "LISTBOX1" ;
                and this.direction == "DOWN" )
            // Move the listbox down one if we can
            if this.parent.listbox1.curSel < this.parent.items.size
               // NOTE: I was using this.parent.listbox.curSel++ here but it
               // had the unfortunate habit of making the listbox 'jump' down
               // and entire page.  I now just press the down arrow.  Ugly,
               // but it works, and looks much nicer.
               KEYB CHR(24)
            else
               this.enabled := false
            endif

         other
            this.enabled := false
         endcase
      CATCH (Exception e)
         this.enabled := false
      ENDTRY
   return
endclass

*------------------------------------------------------------------------------

function FindPath( cDbaseName )
LOCAL nDbaseLen
LOCAL x
LOCAL cSekvence
LOCAL BDEpath
 
nDbaseLen  = len( cDbaseName )
 
if len(cDbaseName) = 0
  RETURN ""      // not using an ALIAS, return NOTHING...
endif

BDEpath = GetBDEPath()

if BDEpath = "**ERROR**"
   RETURN "**ERROR**"
endif

f = new FILE()
 
TRY
f.open( BDEpath, "R" )
 
   x = 15
   cSekvence = ""
 
   // Find Database name in config file
   do while ( cSekvence # cDbaseName .and. .not. f.eof() )
      x++
      f.seek( x, 0 )
      cSekvence := f.gets( nDbaseLen, chr( 0x00 ) )
   enddo
 
   if empty(cSekvence)
     msgbox('Cannot locate database '+cDbaseName+' in BDE configuration file','Looking for BDE Alias',48)
     f.close()
     RETURN "**ERROR**"
   else
 
     // add len of database name + 27 empty places to start before PATH
     x += ( nDbaseLen + 27 )
 
     // Find "PATH" in config file
     do while ( cSekvence # "PATH" .and. .not. f.eof() )
        x++
        f.seek( x, 0 )
        cSekvence := f.gets( 4, chr( 0x00 ) )
     enddo
 
     // add len of "PATH" (4) + 3 empty places
     f.seek( x+7, 0 )
 
     // here is a path for database
     cSekvence = f.gets( 100, chr( 0x00 ) )
   endif
CATCH (exception e) 
   msgbox('Cannot open BDE configuration file: '+BDEpath,'Looking for BDE Alias',48)
   f.close()      
   RETURN "**ERROR**"
ENDTRY 
 
f.close()      
RETURN cSekvence + "\"
 
*------------------------------------------------------------------------------

FUNCTION GetBDEpath()
LOCAL openKey
LOCAL openSubKey
LOCAL nKey
LOCAL nOK
LOCAL nLen
LOCAL nType
LOCAL cData

openKey    = HKEY_LOCAL_MACHINE
openSubKey = "SOFTWARE\Borland\Database Engine"

// OPEN KEY
nKey = 0            // handle of new key 
nOK = RegOpenKeyEx( openKey, openSubKey, 0, KEY_ALL_ACCESS, nKey )
if not (nOK == ERROR_SUCCESS) 
   msgbox( "Cannot open BDE Primary KEY in registry",;
           "Error Finding BDE Path", 64)
   RegCloseKey( nKey )
   return "**ERROR**"
endif

// READ SUBKEY VALUE
nOK   = 0 
nType = 0
nLen  = 80
cData = REPLICATE(" ", 80) 
nOK   = RegQueryValueEx( nKey, "CONFIGFILE01", 0, nType, cData, nLen ) 
if not (nOK == ERROR_SUCCESS)
   msgbox( "Cannot open BDE Path KEY in registry",;
           "Error Finding BDE Path", 64)
   RegCloseKey( nKey )
   return "**ERROR**"
endif

// CLOSE KEY
RegCloseKey( nKey )

RETURN (cData)

*------------------------------------------------------------------------------

FUNCTION question (cText)

f = new questionForm()
f.questiontext.text = cText
f.readModal()

RETURN iif(f.gooddata = true,f.answerentryfield.value,"")

*------------------------------------------------------------------------------

class questionForm of FORM
   with (this)
      onOpen = class::FORM_ONOPEN
      scaleFontBold = false
      height = 4.3636
      left = 103.7143
      top = 14.1818
      width = 41.4286
      text = "Input Information"
      topMost = true
      autoCenter = true
      mdi = false
      smallTitle = true
      maximize = false
      minimize = false
   endwith


   this.PUSHBUTTON1 = new PUSHBUTTON(this)
   with (this.PUSHBUTTON1)
      onClick = {;form.gooddata = true;form.close( form.answerentryfield.value )}
      height = 1.0909
      left = 7.8571
      top = 0.9091
      width = 15.2857
      text = "hidden"
      default = true
      value = false
   endwith


   this.QUESTIONTEXT = new TEXT(this)
   with (this.QUESTIONTEXT)
      height = 1.9091
      left = 0.7143
      top = 0.4545
      width = 39.7143
      alignVertical = 1	// Middle
      alignHorizontal = 1	// Center
      text = "Enter your question here"
   endwith


   this.ANSWERENTRYFIELD = new ENTRYFIELD(this)
   with (this.ANSWERENTRYFIELD)
      height = 1.1818
      left = 0.4286
      top = 2.9091
      width = 40.2857
      value = ""
      function = "!"
   endwith


   function form_onOpen
      form.gooddata = false
      form.answerentryfield.setfocus()

      return
endclass

*------------------------------------------------------------------------------

