//--------------------------------------------------------------------
//--------------------------------------------------------------------
//--------------------------------------------------------------------

// Non visual object class line
class rmAlias of Timer() custom

// Visual form object class line
// class rmAlias(oP) of PaintBox(oP) custom


   this.version = "1.07.0520a"

   /*

   rmAlias.cc

   A custom control which provides the ability to check for the existence of an
   alias, create and delete an alias, and many other alias related functions.

   This control should be used in conjunction with rmBDEConfig.cc which will
   help to ensure that other BDE settings are as they should be such that
   the expected behaviour is realised when using an alias.

   This control can also be used as a visual form object by editing the class
   definition line at the very top.


   For         :  dBASE Plus 2.5 onwards / Win200* / XP


   Authors     :  Romain Strieff        - original code
                  Gary White            - additions
                  Ken Mayer             - additions
                  Roland Wingerter      - additions
                  Ronnie MacGregor      - additions, substitutions,
                                          editing and repackaging
                  Marc Van den Berghen  - additions
                  Andrew Shimmin        - additions

   Contact     :  rmAliasFeedback@dBASEdeveloper.co.uk

   Copyright   :  Ronnie MacGregor and all Authors - All Rights Reserved.

   Disclaimer  :  It is the duty of the user of this code to determine its
                  suitability for use.

                  The use of this code signifies agreement to the following
                  condition of use.

                  In no circumstance will the author accept any responsibility
                  for any consequence arising from the use of this code.

   Acknowledgements :

      A considerable amount of the base code in this control has been adapted
      from BDEAlias.cc and BDEAliasEx.cc from the dUFLP. The original authors
      are listed above.

      In addition :

         Marc Van den Berghen - Struct code.
         Vic McClung - RestartBDE() code.
         Rich Muller - IsUserAdmin() code.
         Romain Streiff - GetComputerName() code.
         Rich Muller - CreateDirTree code.



   Revisions:

      2007/05/11

         Marc Van den Berghen made various changes to this control so that
         correct functionality is provided when diacritical characters are
         involved. Quality assurance of this functionality provided by
         Ivar B. Jessen and Roland Wingerter.


      2007/05/12

         Ronnie MacGregor

         DoesAliasPathExist(cFullPath) function added.

            This allows the developer to check for the existence of a directory
            path which an Alias is going to try and use.

         CreateAliasPath() function added.

            This allows the developer to create a non-existent directory in
            preparation for use by an Alias.


      2007/05/12

         Ronnie MacGregor

         GetTableList() updated to proper(table names).

      2011/04/19

         Andrew Shimmin

         rmAlias class:

         Subclassed non-visual version from the timer class.

         Added the pre-processor constant sesCFGMODE2 and its setting constants.
         Added DbiGetCurrSession(), DbiGetCfgMode2Prop() and DbiSetProp() API Externs.
         Added GetCurrentSessionHandle() method.

         Updated DeleteAlias() to support dBPLUS >= v2.7.

         StructBase class:

         Added GetLogical() and SetLogical() methods.

         Modified SetAnsiString() to reset the whole string before setting the
         string value. This is required when a structure string member is re-used
         with a smaller string than its predecessor.

   Examples:

      Creating or updating an alias :

      Object approach :

         set procedure to rmAlias.cc additive
         o = new rmAlias()
         if o.DoesAliasExist("MYALIAS") == false
            o.CreateDbaseAlias("MYALIAS", cFullDataPath)
         else
            if not upper(o.GetAliasPath("MYALIAS")) == cFullDataPath
               o.DeleteAlias("MYALIAS")
               o.CreateDbaseAlias("MYALIAS", cFullDataPath)
            endif
         endif


      Form Object approach :

         set procedure to rmAlias.cc additive
         form.rmAlias = new rmAlias()
         if form.rmAlias.DoesAliasExist("MYALIAS") == false
            form.rmAlias.CreateDbaseAlias("MYALIAS", cFullDataPath)
         else
            if not upper(form.rmAlias.GetAliasPath("MYALIAS")) == cFullDataPath
               form.rmAlias.DeleteAlias("MYALIAS")
               form.rmAlias.CreateDbaseAlias("MYALIAS", cFullDataPath)
            endif
         endif


      Example of creating an InterBase alias :

         set procedure to rmAlias.cc additive
         o = new rmAlias()

         cParams = "c:\MyTest;" ;
                 + "SERVER NAME:c:\MyTest\testdata.gdb;" ;
                 + "USER NAME:SYSDBA;" ;
                 + "LANGDRIVER:'ascii' ANSI;" ;
                 + "SQLQRYMODE:SERVER;" ;
                 + "SQLPASSTHRU MODE:SHARED NOAUTOCOMMIT"

         o.createAlias("MYINTERBASE", cParams, "INTRBASE")

*/


//--------------------------------------------------------------------
//--------------------------------------------------------------------
//--------------------------------------------------------------------

   // -------------------------------------

   // Standard properties
   try
      this.visible := false
      if form.metric == 6 // pixels
         this.width := 20
         this.height := 20
      endif
   catch(exception e)
      // Based on timer class not paintbox
   endtry

   // -------------------------------------

   // Constants

   #define DBIERR_NONE               0

   #define sesCFGMODE2               0x0002000C                               // new CFGMode (virtual, persistent, session)

   #define cfgmNone                    0x00
   #define cfgmVirtual                 0x01
   #define cfgmPersistent              0x02
   #define cfgmSession                 0x04
   #define cfgmAll                     cfgmVirtual+cfgmPersistent+cfgmSession

   // -------------------------------------

   // API Externs

   if type("DbiAddAlias") # "FP"
      extern cint DbiAddAlias(;
         cHandle, ;
         cstring, ;
         cstring, ;
         cstring, ;
         clogical ;
         );
         idapi32
   endif

   if type("DbiDeleteAlias") # "FP"
      extern cint DbiDeleteAlias(;
         cHandle, ;
         cstring ;
         );
         idapi32
   endif

   if type('DbiExit') # "FP"
      extern cword DbiExit(;
         cvoid ;
         );
         idapi32.dll
   endif

   if type("DbiGetDatabaseDesc") # "FP"
      extern cInt DbiGetDatabaseDesc(;
         cptr, ;
         cptr ;
         );
         idapi32
   endif

   if type("DbiGetDirectory") # "FP"
      extern cInt DbiGetDirectory(;
         cHandle, ;
         clogical, ;
         cptr ;
         );
         idapi32
   endif

   if type("DbiGetSysConfig") # "FP"
      extern cInt DbiGetSysConfig(;
         cPTR ;
         );
         idapi32
   endif

   if type('DbiInitFn') # "FP"
      extern cword DbiInitFn(;
         cword, ;
         cptr ;
         );
         idapi32.dll
   endif

   if type("DbiFormFullName") # "FP"
      extern cint DbiFormFullName(;
         cHandle, ;
         cString, ;
         cString, ;
         cString  ;
         );
         idapi32
   endif

   // Andrew Shimmin

   if type([DbiGetCurrSession]) # [FP]
      extern cushort DbiGetCurrSession(;
         cptr cLong  ;
         );
         idapi32
   endif

   if type([DbiGetCfgMode2Prop]) # [FP]
      extern cushort DbiGetCfgMode2Prop(;
         chandle,    ;
         culong,     ;
         cptr culong,;
         cushort,    ;
         cptr cushort;
         );
         idapi32 from [DbiGetProp]
   endif

   if type([DbiSetProp]) # [FP]
      extern cushort DbiSetProp(;
         chandle, ;
         clong,   ;
         clong    ;
         );
         idapi32 from [DbiSetProp]
   endif

   // -------------------------------------

   function DoesAliasExist(cDatabaseName)

      /*

      Romain Strieff

      February 10, 1999

      Modified by Gary White March 29, 1999

      Edited by Ronnie MacGregor 2006/03/13

      */

      local oDB, aDB, lFound, nCount

      lFound = false

      oDB = new database()
      aDB = oDB.getschema("Databases")
      release object oDB

      for nCount = 1 to aDB.size
         if upper(cDatabaseName) == upper(aDB[nCount])
            lFound := true
            exit
         endif
      endfor

   return lFound

   // -------------------------------------

   function DoesAliasConnect(cAliasName, cTableName, cFieldName, cLoginName, cPassword)

      /*

      Ronnie MacGregor

      2006/03/13

      Returns a boolean value of success connecting
      to the database referenced by the alias.

      The first two parameters are required.

      If opening a database table which contains any field of a type which
      cannot be handled by the BDE then it is important to pass the FieldName
      of a field of an acceptable type as the third parameter to allow the
      table to be opened.

      If opening a database which requires a login, the fourth and fifth
      parameters should be passed in order that the user does not get a
      log in dialogue.

      */

      local oDB, bSuccess

      private cMacro, q

      bSuccess = false

      // Open Database
      try

         oDB = new database()
         oDB.DatabaseName = trim(cAliasName)
         if not empty(argvector(4))
            oDB.LoginString := trim(cLoginName) + "/" + trim(cPassword)
         endif
         oDB.active := true

      catch (exception e)
         // Failed to open database
      endtry

      // Open query
      try

         q = new query()
         q.database := oDB
         q.requestlive := false
         if empty(argvector(3))
            cMacro = "q.sql = [Select * from " + trim(cTableName) + "]"
            &cMacro.
         else
            cMacro = "q.sql = [Select " + trim(cFieldName) + " from " + trim(cTableName) + "]"
            &cMacro.
         endif
         q.active = true

         // If we get here then we have a working connection
         bSuccess := true

         q.active = false
         release object q
         q := null

      catch (exception e)
         // Failed to open query
      endtry

      // Close database (if it opened)
      try

         oDB.active := false
         release object oDB
         oDB := null

      catch (exception e)
         // Failed to close database
      endtry

   return bSuccess

   // -------------------------------------

   function DoesTableExist(cAliasName, cTableName, cLoginName, cPassword)

      /*

      Ronnie MacGregor

      2006/03/13

      Returns a boolean value indicating whether the Table exists in the
      database referred to by the alias.

      The first two parameters are required.

      If opening a database which requires a login, the third and fourth
      parameters should be passed.

      */

      local oDB, bRet

      oDB = new database()
      oDB.DatabaseName = trim(cAliasName)
      if not empty(argvector(4))
         oDB.LoginString := trim(cLoginName) + "/" + trim(cPassword)
      endif
      oDB.active := true
      bRet = oDB.TableExists(trim(cTableName))
      oDB.active := false
      release object oDB
      oDB := null

   return bRet

   // -------------------------------------

   function GetDatabaseType(cAliasName)

      /*

      Ronnie MacGregor

      2006/03/13

      Returns the type of the database.

      A return value of "STANDARD" covers both dBASE and Paradox tables.

      */

      local o, cReturn

      o = new BDEdbInfo()
      o.SetAlias(cAliasName)
      cReturn = o.GetType()
      o.Release()

   return cReturn

   // -------------------------------------

   function GetAliasPath(cAliasName)

      /*

      Ronnie MacGregor

      2006/03/13

      Returns the database path if there is one.

      Usually a true database server does not have a path.
      Usually a non server database will have a path.

      The DatabaseDir function in the dUFLP based on code by Romain Strieff
      and Gary White fails to return the path if the path starts with "\\".

      It is also using the API call "DbiFormFullName" which is a roundabout
      way to get the path when DbiGetDirectory will give a direct result which
      also works with UNC paths.

      There is also a weakness in the method of opening a dBASE database object
      just to get a path. This means that if we try to do this with a server
      type database, the user of our app is presented with a log in dialogue.
      There is now a new GetDatabaseType() function in this class which should
      mean that this above scenario could now be avoided.

      A better method is to get the path information using the BDE API call
      DbiGetDatabaseDesc.

      */

      local o, cReturn

      o = new BDEdbInfo()
      o.SetAlias(cAliasName)
      cReturn = o.GetPath()
      o.Release()

   return cReturn

   // -------------------------------------

   function GetAliasHelpText(cAliasName)

      /*

      Ronnie MacGregor

      March 2006

      Returns the "Help Text" for the Alias if there is any.

      */

      local o, cReturn

      o = new BDEdbInfo()
      o.SetAlias(cAliasName)
      cReturn = o.GetHelpText()
      o.Release()

   return cReturn

   // -------------------------------------

   function DeleteAlias(cAliasName)

      /*

      Romain Strieff

      Deletes an existing Alias

      2007/05/11  Marc Van den Berghen

         Changed to use oem() function in place of unicode functions to
         provide correct behaviour with diacritical characters.

      2011/04/19  Andrew Shimmin

         Added dBPLUS v2.7 support.

      */

      local bSuccess,hSession,nSesCfgMode2

      bSuccess = false
      hSession = 0
      nSesCfgMode2 = 0

      do case
         case  class::IsDbaseUACSupported()                                   // >= v2.7
               if class::GetCurrentSessionHandle(hSession)
                  if (DbiGetCfgMode2Prop(hSession,sesCFGMODE2,nSesCfgMode2,null,null) == DBIERR_NONE)
                     if (DbiSetProp(hSession,sesCFGMODE2,cfgmPersistent) == DBIERR_NONE)
                        bSuccess := (DbiDeleteAlias(null,oem(cAliasName)) == DBIERR_NONE)
                        DbiSetProp(hSession,sesCFGMODE2,nSesCfgMode2)
                     endif
                  endif
               endif
         otherwise                                                            // < v2.7
               bSuccess := (DbiDeleteAlias(null,oem(cAliasName)) == DBIERR_NONE)
      endcase

      return bSuccess

   // -------------------------------------

   function CreateAlias(cAliasName, cFullPath, cTableFormat)

      /*

      Romain Strieff

      September 21, 1998

      cAliasName = the Alias name referring to the the database

      cPath      = the full path to the database

      2007/05/11 Marc Van den Berghen
         Changed to use oem() function in place of unicode functions to
         provide correct behaviour with diacritical characters.

      */

      local bSuccess

      if argCount() < 3
         cTableFormat = "DBASE"
      endif

      bSuccess = DbiAddAlias(;
                    null, ;
                    oem(cAliasName), ;
                    oem(cTableFormat), ;
                    oem("PATH:" + cFullPath), ;
                    TRUE ;
                    );
                    == DBIERR_NONE

   return bSuccess

   // -------------------------------------

   function CreateDbaseAlias(cAliasName, cFullPath)

      /*

      Ronnie MacGregor

      2006/03/05

      dBASE specific function.

      Derived from Romain Strieff's CreateAlias code.

      2007/05/11 Marc Van den Berghen
         Changed to use oem() function in place of unicode functions to
         provide correct behaviour with diacritical characters.

      */

      local bSuccess

      bSuccess = DbiAddAlias(;
                    null, ;
                    oem(cAliasName), ;
                    oem("DBASE"), ;
                    oem("PATH:" + cFullPath), ;
                    TRUE ;
                    );
                    == DBIERR_NONE

   return bSuccess

   // -------------------------------------

   function CreateAccessAlias(cAliasName, cFullPath)

      /*

      Roland Wingerter, 24 Sept. 2004

      2006/03/05
      Ronnie MacGregor
      Turned into Access specific function.

      2007/05/11 Marc Van den Berghen
         Changed to use oem() function in place of unicode functions to
         provide correct behaviour with diacritical characters.

      */

      local bSuccess

      bSuccess = DbiAddAlias(;
                    null, ;
                    oem(cAliasName), ;
                    "MSACCESS", ;
                    oem("DATABASE NAME:" + cFullPath), ;
                    TRUE ;
                    );
                    == DBIERR_NONE

   return bSuccess

   // -------------------------------------

   function CreateMSSQLAlias(cAliasName, cServerName, cDatabaseName)

      /*

      Roland Wingerter  January 6, 2004

      2006/03/05
      Edited by Ronnie MacGregor

      N.B.
      To ensure expected operation with an MSSQL alias,
      ensure that the BDE Init and MSSQL section properties
      are updated with rmBDEConfig.cc

      2007/05/11 Marc Van den Berghen
         Changed to use oem() function in place of unicode functions to
         provide correct behaviour with diacritical characters.

      */

      local bSuccess, cParams

      cParams = "SERVER NAME:" + cServerName  + ";" ;
              + "DATABASE NAME:" + cDatabaseName

      bSuccess = DbiAddAlias(;
                    null, ;
                    oem(cAliasName), ;
                    "MSSQL", ;
                    oem(cParams), ;
                    TRUE ;
                    );
                    == DBIERR_NONE

   return bSuccess

   // -------------------------------------

   function GetBDEConfigFileName()

      /*

      Romain Strieff

      February 2, 1999

      Returns the name of the currently used BDE Configuration file.

      Minor code fix by Lane Coddington

      2007/05/11 Marc Van den Berghen
         Changed to use structbase getansistring() function in place of unicode
         functions to provide correct behaviour with diacritical characters.

      */

      local cRet, cBuffer

      cBuffer = new structbase(256)
      DbiGetSysConfig(cBuffer)
      cRet = cBuffer.getansistring( 53, 128)
      cbuffer.release()

   return cRet

   // -------------------------------------

   function GetLocalShare()

      /*

      Romain Strieff

      February 2, 1999

      Returns a logical true/false based on
      the localShare setting of the BDE.
      Note: this uses fromUnicode() below ...

      January 29, 2002
      Fred van der Meulen
      Bugfix to the creation of the return value

      2007/05/11 Marc Van den Berghen
         Changed to avoid the use of unicode function.

      */

      local bSuccess, cBuffer

      cBuffer = space(256)
      DbiGetSysConfig(cBuffer)
      bSuccess = asc(left(cBuffer, 1)) == 1

   return bSuccess

   // -------------------------------------

   function GetAliasList()

      /*

      Ken Mayer

      September 12, 2002

      This method returns a single-column array of
      Databases in a BDE Alias which can be used
      by the developer to do whatever is needed
      from there.

      2006/03/13
      Rewritten by Ronnie MacGregor to run silently,
      to return false if fails, and sort the list alphabetically.

      */

      local oDB, aSchema

      try
         oDB = new database()
         aSchema = oDB.getSchema("DATABASES")
         aSchema.sort()
         release object oDB
         oDB := null
         return aSchema
      catch( exception e )
         return false
      endtry

   return

   // -------------------------------------

   function GetTableList(cAliasName, cLoginName, cPassword)

      /*

      Ronnie MacGregor

      2006/03/13

      Returns a single-column array of tables
      in the database referred to by the alias.

      */

      local oDB, aTables, nCount, nLen

      try
         oDB = new database()
         oDB.DatabaseName := trim(cAliasName)
         if not empty(argvector(2))
            oDB.LoginString := trim(cLoginName) + "/" + trim(cPassword)
         endif
         oDB.active := true
         aTables = oDB.getSchema("TABLES")
         release object oDB
         oDB := null
         for nCount = 1 to aTables.size
            if rat(".",aTables[nCount])>0
               nLen = rat(".",aTables[nCount])-1
               aTables[nCount] := proper(left(aTables[nCount],nLen))+right(aTables[nCount],len(aTables[nCount])-nLen)
            else
               aTables[nCount] := proper(aTables[nCount])
            endif

         next
         aTables.sort()
         return aTables
      catch( exception e )
         return false
      endtry

   return

   // -------------------------------------

   function GetViewsList(cAliasName, cLoginName, cPassword)

      /*

      Ronnie MacGregor

      2006/03/13

      Returns a single-column array of Views
      in the database referred to by the alias.

      */

      local oDB, aViews

      try
         oDB = new database()
         oDB.DatabaseName := trim(cAliasName)
         if not empty(argvector(2))
            oDB.LoginString := trim(cLoginName) + "/" + trim(cPassword)
         endif
         oDB.active := true
         aViews = oDB.getSchema("VIEWS")
         aViews.sort()
         release object oDB
         oDB := null
         return aViews
      catch( exception e )
         return false
      endtry

   return

   // -------------------------------------

   function GetProceduresList(cAliasName, cLoginName, cPassword)

      /*

      Ronnie MacGregor

      2006/03/13

      Returns a single-column array of stored
      procedures in the database referred to by the alias.

      */

      local oDB, aSPs

      try
         oDB = new database()
         oDB.DatabaseName := trim(cAliasName)
         if not empty(argvector(2))
            oDB.LoginString := trim(cLoginName) + "/" + trim(cPassword)
         endif
         oDB.active := true
         aSPs = oDB.getSchema("PROCEDURES")
         aSPs.sort()
         release object oDB
         oDB := null
         return aSPs
      catch( exception e )
         return false
      endtry

   return

   // -------------------------------------

   function RestartBDE()

      /*

      Vic McClung


      March 2006
      Rewritten by Ronnie MacGregor to return boolean success.

      */

      local bSuccess

      bSuccess = false

      close databases

      if DbiExit() == DBIERR_NONE
         if DbiInitFn(300, null) == DBIERR_NONE
            bSuccess := true
         endif
      endif

   return bSuccess

   // -------------------------------------

   function GetComputername()

      // Romain Streiff

      #define MAX_COMPUTERNAME_LENGTH    15

      local xBuff, nLen

      if type("GetComputerName") # "FP"
         extern CLOGICAL GetComputerName(;
            CSTRING, ;
            CPTR CULONG ;
            ) Kernel32 ;
         from "GetComputerNameA"
      endif

      nLen = MAX_COMPUTERNAME_LENGTH + 1

      xBuff = Replicate(Chr(0), nLen)

      if GetComputerName(xBuff, nLen)
         return Left(xBuff.String, nLen)
      else
         return false
      endif

   return

   // -------------------------------------

   function IsDbaseUACSupported()

      // Andrew Shimmin

      return (type([_app.useUACPaths]) # [U])

   // -------------------------------------

   function IsBitSet(nMask,nValue)

      // Andrew Shimmin

      return (bitAnd(nMask,nValue) = nMask)

   // -------------------------------------

   function IsUserAdmin()

      // Rich Muller

      local oNet, TestShare, sComputerName

      // Try API method first
      try
         if type('IsUserAnAdmin') # 'FP'
            extern CLOGICAL IsUserAnAdmin() shell32
         endif
         return IsUserAnAdmin()
      catch(exception e)
      endtry

      // Try WSH second
      sComputerName = this.GetComputerName()
      if empty(sComputerName)
         try
            oNet = new OleAutoClient("WScript.Network")
            ComputerName = oNet.ComputerName
            release object oNet
         catch(exception e)
            ComputerName = getEnv("COMPUTERNAME")
         endtry
      endif
      TestShare = "\\" + sComputerName + "\admin$\*.*"

   return (new array().dir(TestShare)>0)

   // -------------------------------------

   function DoesAliasPathExist(cFullPath)

      // Returns a boolean value

   return (len(FUNIQUE(cFullPath + "\")) > 0)

   // -------------------------------------

   function CreateAliasPath

      /*

      Renamed version of MakeDirTree()

      Rich Muller - www.autotraker.com

      This function will build an entire directory tree
      just pass the entire tree and it will build it.

      cDirTree = "c:\temp2\temp2\temp3\temp4\temp5\temp6"
      MakeDirTree(cDirTree)

      Returns true if the directory exists if it has build it.

      Can use either imageHlp.dll or dbgHelp.dll (system32 directory).
      Since some versions of imageHelp.dll may not support this function,
      we are putting it in a try/catch/etc.

      */

      parameters cDirTree

      local cDir

      cDir = iif(right(cDirTree, 1) == "\", cDirTree, cDirTree + "\")

      if type("MakeSureDirectoryPathExists")<>"FP"
         try
            extern cLogical MakeSureDirectoryPathExists(cString) "IMAGEHLP.DLL"
         catch(exception e)
         endtry
      endif

      if type("MakeSureDirectoryPathExists")<>"FP"
         extern cLogical MakeSureDirectoryPathExists(cString) "dbgHelp.dll"
      endif

   return MakeSureDirectoryPathExists(cDir)

   // -------------------------------------

   function GetCurrentSessionHandle(nHandle)

      // Andrew Shimmin

      local bSuccess

      nHandle := 0

      bSuccess = DbiGetCurrSession(;
                    nHandle ;
                    );
                    == DBIERR_NONE

      return bSuccess

endclass

//--------------------------------------------------------------------
//--------------------------------------------------------------------
//--------------------------------------------------------------------

Class BDEdbInfo of object

   /*

   Ronnie MacGregor and Marc Van den Berghen

   March 2006

   Get Alias information out of the BDE using the BDE API.

   */


   // Constants

   #define DBIERR_NONE               0
   #define DBIERR_OBJNOTFOUND     8712


   // API Externs

   if type("DbiGetDbDesc") # "FP"
      extern cInt DbiGetDbDesc(;
         clong, ;
         cptr ;
         );
         idapi32 ;
      from "DbiGetDatabaseDesc"
   endif


   // Custom properties
   this.Error  = 0
   this.szName = new StringPointer()
   this.dbDesc = new DBDescStruct()
   this.dbName = ""
   this.dbPath = ""
   this.dbText = ""
   this.dbType = ""

   // -------------------------------------

   function SetAlias(cAliasName)

      local szn, bSuccess

      bSuccess = false

      // Get a pointer to the alias string
      szn = val(this.szName.pstring(cAliasName))

      // Get the BDE information
      this.Error = DbiGetDbDesc(szn, this.dbDesc)

      if this.Error == DBIERR_NONE

         // Set custom properties
         this.dbName := this.dbDesc.szName()
         this.dbPath := this.dbDesc.szPhyName()
         this.dbText := this.dbDesc.szText()
         this.dbType := this.dbDesc.szType()

         bSuccess := true

      endif

   return bSuccess

   // -------------------------------------

   function GetName()

      /*

      Returns the Alias name.

      This might seem a bit pointless since we need the alias
      name to use this class, but it does return the information
      contained in the BDE.

      */

   return this.dbName

   // -------------------------------------

   function GetPath()

      /*

      Returns the path if there is one.

      Usually a true database server does not have a path.
      Usually non server databases will have a path.

      */

   return this.dbPath

   // -------------------------------------

   function GetHelpText()

      // Returns the "Help Text" if there is any.

   return this.dbText

   // -------------------------------------

   function GetType()

      // Returns the type of the Database

      local cType

      do case

         case this.dbType == "STANDARD"
            cType = (this.dbType)

         case this.dbType == "MSSQL"
            cType = (this.dbType)

         case this.dbType == "SQL Server"
            cType = (this.dbType)

         case this.dbType == "Microsoft Access Driver (*.mdb)"
            cType = "Access"

         // Other types to be added here as required

         otherwise
            cType = (this.dbType)

      endcase

   return cType

   // -------------------------------------

   function Release()

      this.szName.release()
      this.dbDesc.release()
      release object this

   return

   // -------------------------------------

endclass


//--------------------------------------------------------------------
//--------------------------------------------------------------------
//--------------------------------------------------------------------

class DBDescStruct of StructBase(358)

   /*

   Marc Van den Berghen

   Structure for the BDEdbInfo class.

   */

   // -------------------------------------

   function szText(c)
   return iif(pcount() == 0, this.Ansistring(32,32), this.ansistring(32, 32, c))

   // -------------------------------------

   function szName(c)
   return iif(pcount() == 0, this.Ansistring(0,32), this.ansistring(0, 32, c))

   // -------------------------------------

   function szPhyName(c)
   return iif(pcount() == 0, this.Ansistring(64,261), this.ansistring(64, 261, c))

   // -------------------------------------

   function szType(c)
   return iif(pcount() == 0, this.ansistring(325,32), this.ansistring(325, 32, c))

   // -------------------------------------

   function release()
      super::release()
   return

   // -------------------------------------

endclass

//--------------------------------------------------------------------
//--------------------------------------------------------------------
//--------------------------------------------------------------------

class StringPointer of StructBase(4)

   /*

   Marc Van den Berghen

   This is not really a structure, but can be used if an external call
   uses a pointer to a string as parameter.

   */

   // Custom properties

   this.p = 0             // Pointer to psztext

   // -------------------------------------

   function pString(c)

      if pcount() == 0
         msgbox("Sorry, not implemented yet", " Error", 0)
         return 0
      endif

   return this.SetText(c, 0, this.p)

   // -------------------------------------

   function Release()

      if this.p # 0
         GlobalFree(this.p)
      endif
      super::Release()

   return

   // -------------------------------------

endclass

//--------------------------------------------------------------------
//--------------------------------------------------------------------
//--------------------------------------------------------------------

Class StructBase(nSize) Of String

   /*

   Marc Van den Berghen

   Base class for all structures.

   From API_Structs.cc

   Revisions:

      2011/04/19  Andrew Shimmin

         Added GetLogical() and SetLogical() methods.

         Modified SetAnsiString() to reset the whole string before setting the
         string value. This is required when a structure string member is re-used
         with a smaller string than its predecessor.
   */

   // -------------------------------------

   // Constants

   #define GMEM_FIXED 0

   // -------------------------------------

   // Parameters

   if empty(nSize)
      nSize=128
   endif
   // Enlarge to WordBorder for Unicode
   nSize = iif((nSize % 2) == 1, nSize + 1, nSize)

   // -------------------------------------

   // Custom Properties

   this.String = Replicate(Chr(0),nSize / 2) // Initialize with 0
   this.SizeOf = nSize                       // size of struct

   // -------------------------------------

   // API Externs

   if type("GlobalAlloc") # "FP"
      extern chandle GlobalAlloc(;
         cuint, ;
         clong ;
         );
         kernel32
   endif

   if type("GlobalFree") # "FP"
      extern chandle GlobalFree(;
         chandle ;
         ) ;
         kernel32
   endif

   if type("GetCurrentProcess") # "FP"
      extern chandle GetCurrentProcess(;
         cvoid ;
         ) ;
         kernel32
   endif

   if type("WriteProcessMemory") # "FP"
      extern clogical WriteProcessMemory(;
         chandle, ;
         cptr, ;
         cstring, ;
         clong, ;
         cptr ;
         ) ;
         kernel32
   endif

   if type("WriteProcessMemoryP") # "FP"
      extern clogical WriteProcessMemoryP(;
         chandle, ;
         cptr, ;
         cptr, ;
         clong, ;
         cptr ) ;
         kernel32 ;
      from 'WriteProcessMemory'
   endif

   // -------------------------------------

   function GetLong(nIndex)

      // Get the long(dword) value

      local n
      n = this.getByte(nIndex) ;
        + bitlshift(this.getByte(nIndex + 1), 8) ;
        + bitlshift(this.getByte(nIndex + 2), 16) ;
        + bitlshift(this.getByte(nIndex + 3), 24)

   return n

   // -------------------------------------

   function SetLong(nIndex, nValue)

      /*
      Set the long value at the desired
      position in the structure
      */

      this.setByte(nIndex, bitand(nValue, 0xFF))
      this.setByte(nIndex + 1, bitrshift(bitand(nValue, 0xFF00), 8))
      this.setByte(nIndex + 2, bitrshift(bitand(nValue, 0xFF0000), 16))
      this.setByte(nIndex + 3, bitrshift(bitand(nValue, 0xFF000000), 24))

   return

   // -------------------------------------

   function Release()

      // cleanup

      this.String = ''
      release object this

   return

   // -------------------------------------

   function Member(nIndex, nValue)

      /*

      The following method is used to overload the member-functions
      of the subclassed structures, so that the same name can be used
      to set or to get the values (by checking the number of parameters)

      */

      if pcount() == 1
         return this.getlong(nIndex)
      else
         this.setlong(nIndex, nValue)
      endif

   return nValue

   // -------------------------------------

   function SetText

      /*

      Windows often uses pointers to strings in structures. The
      following method was specially designed to enable the
      dbase developper to use such structures. It allocates memory
      to hold the string and gets the address of this memory
      block from the OS.

      2007/05/11 Marc Van den Berghen
         Changed to use oem() function to ensure correct behaviour with
         diacritical characters.

      */

      // Pass pointer as reference so it can be changed

      parameters c, n, h

      local apierror, cResult, hprocess

      cResult = ""

      if type("c")="U"

         return cResult

      else

         if h # 0

            // release the old memory-block so that
            // it can be reused later (and to
            // avoid memory-leaks)

            cResult = GlobalFree(h)

         endif

         if type("c") == "C"

            h = globalalloc(GMEM_FIXED, len(c) + 1) // Get the address
            if h=0
               msgbox("Memory allocation failed", " Fatal Error", 0)
            else
               hprocess = GetCurrentProcess()
               // Write the string to the reserved memory-location
               apierror = WriteProcessMemory(hprocess, h, oem(c + chr(0)), len(c) + 1, 0)
               if not apierror == true
                  msgbox("Pointer creation failed", " Fatal Error", 0)
               else
                  cResult = str(h)
                  this.setlong(n, h)
               endif
            endif
         else
            this.setlong(n, c)
         endif
      endif

      // Return either empty string (on error) or pointer to the string

   return cResult

   // -------------------------------------

   function GetText(n, h)

      /*

      This method is not fully implemented yet !!!!!!!
      (the value of the string at the position pointed to is not
      yet returned)

      */

      local cResult
      cResult = ""
      h = this.getlong(n)

   return

   // -------------------------------------

   function AnsiString(nIndex, nMax, cValue)

      /*

      The following function is used to get and set string-values
      in a structure

      !!! Attention : it handles only ANSI-Characters (singlebyte)
      Unicode not yet implemented

      nMax : must contain the maximum space accorded to the specific
            string-member (used to avoid overwriting of the other members)

      */

      if pcount() == 2
         return this.GetAnsiString(nIndex, nMax)
      else
         this.SetAnsiString(nIndex, nMax, cValue)
      endif

   return cValue

   // -------------------------------------

   function SetAnsiString(nIndex, nMax, cValue)

      /*

      2007/05/11 Marc Van den Berghen

         Changed to use oem() function to ensure correct behaviour with diacritical characters.

      2011/04/19 Andrew Shimmin

         Changed to reset the whole string before before setting the string value.

         This is required when a structure string member is reused with a smaller
         string than its predecessor.

         The TaskBarTray.cc can do the above with the szTip member of the _NotifyIconData structure.

      */

      local i,cReset

      cReset = replicate(oem(chr(0)),nMax)

      for i = 0 to nMax-1
         this.SetByte(nIndex+i,asc(cReset.charAt(i)))                         // reset string
      endfor

      cvalue = oem(cvalue)
      nMax = iif(len(cvalue) < nMax-1, len(cValue) - 1, nMax - 2)
      for i = 0 to nMax
         this.SetByte(nIndex + i, asc(cValue.charAt(i)))
      endfor

   // -------------------------------------

   function GetAnsiString(nIndex, nMax)

      // Assuming null-termination

      local cValue, zw

      cvalue = ''

      for i = 0 to nMax-1
         zw = this.GetByte(nIndex + i)
         if zw # 0
            cValue += chr(zw)
         else
            i = nMax
         endif
      endfor

   return ansi(cValue)

   // -------------------------------------

   function WordMember(nIndex, nValue)

      // Get and set two-byte values (words)

      if pCount() == 1
         return this.GetWord(nIndex)
      else
         this.SetWord(nIndex, nValue)
      endif

   return nValue

   // -------------------------------------

   function GetWord(nIndex)

      // get the word value

      local n

      n = this.GetByte(nIndex) ;
        + bitlshift(this.GetByte(nIndex + 1), 8)

   return n

   // -------------------------------------

   function SetWord(nIndex,nValue)

      // set the word value at the desired

      this.SetByte(nIndex, bitand(nValue, 0xFF)) // position in the structure
      this.SetByte(nIndex + 1, bitrshift(bitand(nValue, 0xFF00), 8))

   return

   // -------------------------------------

   function GetLogical(nIndex)

      // Andrew Shimmin

      local n,lValue

      // get the word value

      n = this.GetByte(nIndex) ;
        + bitlshift(this.GetByte(nIndex + 1), 8)

      lValue = (n = true)

   return lValue

   // -------------------------------------

   function SetLogical(nIndex,lValue)

      // Andrew Shimmin

      local nValue

      nValue = iif(lValue,1,0)

      // set the word value at the desired location

      this.SetByte(nIndex, bitand(nValue, 0xFF))                              // position in the structure
      this.SetByte(nIndex + 1, bitrshift(bitand(nValue, 0xFF00), 8))

   return

   // -------------------------------------

   function FillStruct(lp_str)

      local hProcess, apiError

      hprocess = GetCurrentProcess()

      apiError = WriteProcessMemoryp(hProcess, this.String, lp_str, this.SizeOf, 0)
      if not apiError == true
         msgbox("Pointer not valid", " Fatal Error", 0)
      endif

   return

   // -------------------------------------

endclass

//--------------------------------------------------------------------
//--------------------------------------------------------------------
//--------------------------------------------------------------------

// eof

